function [p_single, p_double] = bootstrap_test(A, B, n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS)
% function [p_single, p_double] = bootstrap_test(A, B, n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS)
% 
% Performs a bootstrap test to check the null hypothesis that A and B
% come from the same distribution. The p that comes out of the boostrap
% process represents the probability of this null hypothesis. If small, A
% and B are likely to come from a different distribution. 
%
% In this bootstrap test, we focus on the *standard deviation* of A and B.
%
% More information on bootstrap tests:
% https://en.wikipedia.org/wiki/Bootstrapping_(statistics)#:~:text=Bootstrapping%20is%20any%20test%20or,broader%20class%20of%20resampling%20methods.&text=This%20technique%20allows%20estimation%20of,statistic%20using%20random%20sampling%20methods.
%
% - A, B: Vectors with numbers representing samples from the two distributions.
% - n_iterations: How often we should sample from the set C, that combines A and B.
% - WITH_RESAMPLING: Whether we should sample with resampling from C.
% - RETAIN_LENGTHS: Whether we should keep the sampled A' and B' the same size 
% as the original A and B.
%
% - p_single: p for a single-sided test (weaker)
% - p_double: p for a double-sided test (stronger)
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('n_iterations', 'var') || isempty(n_iterations))
    n_iterations = 10000;
end

if(~exist('WITH_RESAMPLING', 'var') || isempty(WITH_RESAMPLING))
    WITH_RESAMPLING = true;
end

if(~exist('RETAIN_LENGTHS', 'var') || isempty(RETAIN_LENGTHS))
    RETAIN_LENGTHS = true;
end

% Determine the standard deviation of the original sample vectors:
diff_std = std(A) - std(B);

% Make a new set C that represents the null hypothesis:
% A and B come from the same distribution (C)
C = [A; B];
N = length(C);
fprintf('Number of samples = %d.\n', N);
% make a vector for the new sampled differences:
diffs = zeros(n_iterations, 1);
% loop over the iterations:
for it = 1:n_iterations
    
    % Make a vector AA and BB by sampling:
    if(WITH_RESAMPLING)
        inds = ceil(rand(N,1) * N);
    else
        inds = randperm(N);
    end
    
    if(RETAIN_LENGTHS)
        inds_AA = inds(1:length(A));
        inds_BB = inds(length(A)+1:end);
    else
        inds_AA = inds(1:floor(N/2));
        inds_BB = inds(floor(N/2)+1:end);
    end
    
    AA = C(inds_AA);
    BB = C(inds_BB);
    
    % determine the sampled difference between the standard deviations:
    diffs(it) = std(AA) - std(BB);
end

% Find out where the original difference lies, compared to the sampled ones
% under the null hypothesis:
[v, i] = sort(diffs);
inds_smaller = find(diffs < diff_std);
inds_larger = find(diffs > diff_std);
frac = min([length(inds_smaller), length(inds_larger)]) / n_iterations;
% fprintf('Single sided, p = %f\n', frac);
p_single = frac;
if(p_single <= 0.001)
    fprintf('p single sided <= 0.001\n');
end

[v, i] = sort(abs(diffs));
inds_larger = find(diffs > abs(diff_std));
frac = length(inds_larger) / n_iterations;
% fprintf('Double sided, p = %f\n', frac);
p_double = frac;
if(p_double <= 0.001)
    fprintf('p double sided <= 0.001\n');
end





