function chart_performance_constant_altitude_changing_attitude_moment(N, SINGLE_SETTING, RATE_MEASUREMENT)
% function chart_performance_constant_altitude_changing_attitude(N, SINGLE_SETTING, RATE_MEASUREMENT)
%
% Function that changes parameters of the model, charting control
% performance for various parameter chanages.
%
% - N: number of simulations
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('N', 'var') || isempty(N))
    N = 5;
end
if(~exist('SINGLE_SETTING', 'var') || isempty(SINGLE_SETTING))
    SINGLE_SETTING = true;
end
if(~exist('RATE_MEASUREMENT', 'var') || isempty(RATE_MEASUREMENT))
    RATE_MEASUREMENT = true;
end

% set parameters for the experiment:
MOMENT = true;
params = set_standard_params_simulate_controller_constant_altitude(MOMENT, RATE_MEASUREMENT);
params.sigma_M = 2.5e-5; % moment noise
params.OFdt = 0.01;
params.sigma_OF = 0.01;
params.delay_OF = 1/params.OFdt;
params.control_ref = [0 0.1];
params.sigma_rate = 0.02; 
params.sigma_T = 0.05; 
params.sigma_rate_msm = 0.001; % rate measurement noise
% delays, all in seconds:
params.delay_rate = 0.01;      % delay in rate input to system
params.delay_rate_k = 0.01;   % delay in rate input to kalman filter
params.delay_T = 0.01;      % delay in thrust input to system
params.delay_T_k = 0.01;   % delay in thrust input to kalman filter
params.delay_moment = 0.01; % delay in moment input to system

% External disturbances:
NO_DISTURBANCE = 0;
THRUST_DISTURBANCE = 1;
HORIZONTAL_DISTURBANCE = 2;
MALICIOUS_V0 = 3;
params.external_disturbance = HORIZONTAL_DISTURBANCE;

% no plots when running the simulator:
params.graphics = false;

% random seed:
rng(2021);

if(SINGLE_SETTING)
    % test a single setting for debugging:
    params.OFdt = 0.01;
    params.delay_OF = 1*params.OFdt;
    params.R = params.R * 1;
    [outliers_percentage, mean_MAE, mean_RT, mean_PO] = get_results_of_setting(N, params)
else
    % investigate sensor and actuation noise separately:
    % noise settings, first row sigma_M, second row sigma_OF:
    noise_settings = [0,    0,    0, 5e-6,   3e-5, 5e-5; ...
        0,  0.01, 0.1, 0,   0,   0];
    OFdts = [1/300, 0.01, 1/50, 1/30, 0.1, 1.0];
    n_settings = size(noise_settings, 2);
    n_OFdts = length(OFdts);
    
    % create matrices for storing results:
    OP = zeros(n_OFdts, n_settings);
    MAE = zeros(n_OFdts, n_settings);
    RT = zeros(n_OFdts, n_settings);
    PO = zeros(n_OFdts, n_settings);
    
    % loop over all settings:
    for s = 1:n_settings
        for o = 1:n_OFdts
            
            % set the sensing and noise characteristics
            params.sigma_M = noise_settings(1, s);
            params.sigma_OF = noise_settings(2, s);
            params.OFdt = OFdts(o);
            params.delay_OF = 1*OFdts(o); % delay is equal to sampling time of OF
            
            % run the simulator N times per setting, accumulating results:
            [outliers_percentage, mean_MAE, mean_RT, mean_PO] = get_results_of_setting(N, params);
            
            % store the results:
            OP(o,s) = outliers_percentage;
            MAE(o,s) = mean_MAE;
            RT(o,s) = mean_RT;
            PO(o,s) = mean_PO;
        end
    end
    
    % save them in a structure:
    results.noise_settings = noise_settings;
    results.settings = cumsum(ones(1,n_settings));
    results.OFdts = OFdts;
    results.OP = OP;
    results.MAE = MAE;
    results.RT = RT;
    results.PO = PO;
    if(RATE_MEASUREMENT)
        post_fix = '_rate';
    else
        post_fix = '';
    end
    save(['results_noise_performance' post_fix], 'results');
    
    % plot the results:
    plot_results_performance_chart(results)
end

function [outliers_percentage, mean_MAE, mean_RT, mean_PO] = get_results_of_setting(N, params)
% function [outliers_percentage, mean_MAE, mean_RT, mean_PO] = get_results_of_setting(N, params)
%
% Get control performance results for a simulation with params over N runs.

verbose = true;
if(N == 1)
    params.graphics = true;
end

% initialize variables for storing results over multiple runs
MAE_GT = zeros(N, 1);
RT = zeros(N,1);
PO = zeros(N,1);


for exp = 1:N
    fprintf('Experiment %d\n', exp);
    metrics = simulate_controller_constant_altitude_changing_attitude_moment(params);
    if(verbose)
        print_metrics(metrics);
    end
    
    MAE_GT(exp) = metrics.MAE_GT;
    RT(exp) = metrics.rise_time;
    PO(exp) = metrics.percentage_overshoot;
end

inds_RT = find(RT < 1000);
inds_PO = find(RT < 1000);
inds = intersect(inds_RT, inds_PO);

outliers_percentage = 100 - 100 * (length(inds) / N);
fprintf('Outliers percentage = %f\n', outliers_percentage);

if(~isempty(inds))
    fprintf('Statistics for inliers: \n');
    fprintf('Avg (MAE, RT, PO) = (%f, %f, %f)\n', mean(MAE_GT(inds)), mean(RT(inds)), mean(PO(inds)));
    fprintf('Std (MAE, RT, PO) = (%f, %f, %f)\n', std(MAE_GT(inds)), std(RT(inds)), std(PO(inds)));
end

mean_MAE = mean(MAE_GT(inds));
mean_RT = mean(RT(inds));
mean_PO = mean(PO(inds));