function [rise_time, percentage_overshoot, mean_absolute_error] = get_performance_characteristics(signal, reference, times, POS_PEAKS)
% function [rise_time, percentage_overshoot, mean_absolute_error] = get_performance_characteristics(signal, reference, times, POS_PEAKS)
% signal = the observed signal that is used for control - only send the
% part after the step!
% reference = a single value after the step
% times = the time in seconds along the signal (make sure it starts at 0 at
% the step)
% POS_PEAKS = look for positive peaks (maxima, in case the reference higher after the step than
% before). This is default. If false, then minima are looked for.
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

T = length(signal);

if(~exist('POS_PEAKS', 'var') || isempty(POS_PEAKS))
    POS_PEAKS = true;
end

% rise time is time till first crossing with the reference:
for t = 1:T-1
    if((signal(t) <= reference && signal(t+1) >= reference) || (signal(t) >= reference && signal(t+1) <= reference) || signal(t) == reference)
        % linear interpolation:
        if(signal(t) ~= signal(t+1))
            alpha = (reference - signal(t+1)) / (signal(t) - signal(t+1));
            rise_time = alpha * times(t) + (1-alpha) * times(t+1);
        else
            % constant value, in principle no crossing
            rise_time = times(t);
        end
        break;
    end
end
if(~exist('rise_time', 'var'))
    % signal does not reach the reference, give a very large value:
    rise_time = 1000;
end

% percentage overshoot absolute value of the first peak divided by the absolute reference value:
try
    if(POS_PEAKS)
        [pks,locs] = findpeaks(signal);
        inds = find(pks > reference);
    else
        [pks,locs] = findpeaks(-signal);
        inds = find(-pks < reference);
    end
    
    if(~exist('inds', 'var') || isempty(inds))
        percentage_overshoot = 1000; % large number
    else
        ind = inds(1);
        ratio = abs(pks(ind)) / abs(reference);
        percentage_overshoot = 100 * (abs(ratio - 1));
    end
catch e
    fprintf('?');
end

% mean absolute error:
mean_absolute_error = mean(abs(signal - reference));

