function investigate_active_oscillations()
% function investigate_active_oscillations()
%
% Runs a simulator three times with different settings: no active
% osillations, and active oscillations at 1 Hz / 10 Hz.
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

% type of model:
MOMENT = false;
RATE_MEASUREMENT = true;
NOISE = true; % without noise, the non oscillation case is completely unobservable
DELAY = false; % if true, just shifts the graph

% set the parameters for simulation:
params = set_standard_params_simulate_controller_constant_altitude(MOMENT, RATE_MEASUREMENT);
params.graphics = false;
params.T = 1.1;
params.OFdt = 1/100;
if(NOISE)
    params.sigma_rate = 0.005;
    params.sigma_rate_msm = 0.01;
    params.sigma_OF = 0.01;
end
if(DELAY)
    params.delay_moment = params.dt;    % delay in moment input to system
    params.delay_rate = params.dt;      % delay in rate input to system
    params.delay_OF = params.OFdt;    % delay in ventral flow measurements
end

% periods of oscillations to test:
periods = [1.0, 0.10];
n_periods = length(periods);
params.active_rate_amplitude = 0.2; % amplitude of the active oscillation in rad/s

% First run the sim without additional oscillations:
results = cell(n_periods+1, 1);
params.ACTIVE_OSCILLATION = false;
[metrics, results{1}] = simulate_controller_constant_altitude_changing_attitude(params);

% Active observability through oscillations:
params.ACTIVE_OSCILLATION = true;
for p = 1:n_periods
    params.period = periods(p);
    [metrics, results{p+1}] = simulate_controller_constant_altitude_changing_attitude(params);
end

% plot the results:
color_1 = [0 0 0.8];
color_2 = [0.8 0 0];
n_colors = length(periods) + 1;
color_step = 1/ (n_colors-1);
ONE_PLOT = true;
figure(); set(gcf,'color','w');
lw = 1.25;
if(ONE_PLOT)
    hold on;
end
lg = cell(n_periods+1,1);
for p = 1:n_periods+1
    if(~ONE_PLOT)
        subplot(n_periods+1, 1, p);
    end
    color_alpha = (p-1)* color_step;
    plot(results{p}.time, results{p}.observability, 'Color', color_alpha * color_1 + (1-color_alpha)*color_2, 'LineWidth', 2);
    if(p == 1)
        lg{p} = 'No oscillations';
    else
        lg{p} = [num2str(round(1/periods(p-1))) ' Hz'];
    end
    if(~ONE_PLOT)
        legend(lg{p});
        xlim([0,1]);
        ylim([0,1.1]);
        xlabel('Time [s]');
        ylabel('Observability degree [-]');
    end
end
if(ONE_PLOT)
    legend(lg);
    xlim([0,1]);
    ylim([0,1.1]);
    xlabel('Time [s]');
    ylabel('Observability degree [-]');
end
set(gca,'FontSize', 16);