function [X_k1_k1, P_k1_k1, i_k1, K_k1] = kalman_filter_constant_alt_changing_att_moment(X, P, Z, m, g, M, I, kd, t1, t2, G, Q, R, flag)
% function [X_k1_k1, P_k1_k1, i_k1, K_k1] = kalman_filter_constant_alt_changing_att_moment(X, P, Z, m, g, M, I, kd, t1, t2, G, Q, R, flag)
%
% Extended Kalman Filter for estimating the state of a constant height,
% varying attitude model with moment commands.
%
% - X: current state estimate
% - P: current covariance matrix
% - Z: measurements
% - m, g: mass, gravity
% - M: commanded moment
% - I, kd: moment of inertia, drag parameter
% - t1, t2: time span
% - G: continuous system matrix for control inputs
% - Q, R: Actuation, sensing variance 
% - flag: whether an optic flow measurement took place at the current time step.
%
% - X_k1_k1, P_k1_k1, i_k1, K_k1: New estimated state, covariance matrix,
% innovation, Kalman gains.
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

Nx = length(X);

% Prediction: odefcn_const_alt_changing_att(t, x, m, g, p, kd, xv)
X_k1_k =  X + odefcn_const_alt_changing_att_moment(1, X, m, g, M, I, kd, [0 0 0 0])*(t2-t1);

v = X_k1_k(1); theta = X_k1_k(2);  thetad = X_k1_k(3); z = X_k1_k(4);

% State transition matrix:
% Linear drag model -kd * v as drag model:
Fx(1, :) = [ -kd/m, g*(tan(theta)^2 + 1), 0, 0];
Fx(2, :) = [     0,                  0, 1, 0];
Fx(3, :) = [     0,                  0, 0, 0];
Fx(4, :) = [     0,                  0, 0, 0];

% Jacobian observation matrix:
if(length(Z) == 1)
    Hx = [-cos(theta)^2/z, (v*sin(2*theta))/ z, 1, (v* cos(theta)^2)/z^2];   %%%%%%%%%
else
    Hx = [-cos(theta)^2/z, (v*sin(2*theta))/ z, 1, (v* cos(theta)^2)/z^2; ...
        0,                0,                  1, 0]; % rate
end

% Discretize 
[Phi_k1_k, Gamma_k1_k] = c2d(Fx, G, t2-t1);

% Covariance matrix of state prediction error
P_k1_k = Phi_k1_k*P*Phi_k1_k' + Gamma_k1_k*Q*Gamma_k1_k';

% Kalman gain
S_k = Hx*P_k1_k*Hx' + R;
K_k1 = P_k1_k*Hx' * pinv(S_k);

if(flag == 1)
    % Measurement update
    if(length(Z) == 1)
        Z_expected = [-v*cos(theta)*cos(theta)/z + thetad];
    else
        Z_expected = [-v*cos(theta)*cos(theta)/z + thetad; ...
            thetad]; % Rate     
    end
    i_k1 = Z - Z_expected;  % Innovation  %%%%%%%%%%
    
    X_k1_k1 = X_k1_k + K_k1*(i_k1);
    
    % Covariance matrix of state estimation error
    P_k1_k1 = (eye(Nx) - K_k1*Hx)*P_k1_k*(eye(Nx) - K_k1*Hx)' + K_k1*R*K_k1'; % Joseph form of the covariance update equation

else
    
    if(length(Z) == 1)
        X_k1_k1 = X_k1_k;
        P_k1_k1 = P_k1_k;
        i_k1 = 0;
    else
        % We do have a rate measurement:
        Z_expected = [-v*cos(theta)*cos(theta)/z + thetad; ...
            thetad]; % Rate
        % we adapt the optical flow measurement to perfectly fit with
        % the expectation. Only the rate can now lead to innovation:
        Z(1) = Z_expected(1);
        i_k1 = Z - Z_expected;      % Innovation  %%%%%%%%%%
        
        % Update the state:
        X_k1_k1 = X_k1_k + K_k1*(i_k1);
        
        % Covariance matrix of state estimation error
        P_k1_k1 = (eye(Nx) - K_k1*Hx)*P_k1_k*(eye(Nx) - K_k1*Hx)' + K_k1*R*K_k1'; % Joseph form of the covariance update equation
    end
end
