function local_observability_analysis(METHOD, v_max, z_max, max_angle, max_rate)
% function local_observability_analysis(METHOD, v_max, z_max, max_angle, max_rate)
%
% Perform a local observability analysis for the basic, constant height
% model.
%
% - METHOD: 1 = random points in state space, 2 = grid over the state space
% - v_max, z_max, max_angle, max_rate: determine the intervals in which the
% state space is sampled for verification. 
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

% two verification methods:
RANDOM = 1; % random points in the state space interval
GRID = 2; % grid over the state space interval

if(~exist('METHOD', 'var') || isempty(METHOD))
    METHOD = GRID;
end
if(~exist('v_max', 'var') || isempty(v_max))
    v_max = 5; % [-max, max]
end
if(~exist('z_max', 'var') || isempty(z_max))
    z_max = 20; % [0, max]
end
if(~exist('max_angle', 'var') || isempty(max_angle))
    max_angle = (0.5-1e-2)*pi; % [-max, max]
end
if(~exist('max_rate', 'var') || isempty(max_rate))
    max_rate = 0.5*pi; % in rad, [-max, max]
end

if(METHOD == GRID)
    % Construct the grid:
    
    % With 9 steps, each interval has 10 values.
    % This leads to 10^4 = 10000 test samples.
    n_steps = 9;
    
    v_step = 2*v_max / n_steps;
    z_step = (z_max-0.1) / n_steps;
    angle_step = 2*max_angle / n_steps;
    rate_step = 2*max_rate / n_steps;
    
    vs = -v_max:v_step:v_max;
    zs = 0.1:z_step:z_max;
    as = -max_angle:angle_step:max_angle;
    ps = -max_rate:rate_step:max_rate;
    
    [V, Z, A, P] = ndgrid(vs, zs, as, ps);
    V = V(:); Z = Z(:); A = A(:); P = P(:);
    
    % number of states / observation vectors that are tested:
    n_tests = length(V);
else
    % number of states / observation vectors that are tested:
    n_tests = 10000;
end

% make symbolic variables:
syms v z m kd g p phi C1 C2 C3 C4;

assume(v, 'real');
assume(z, 'real');
assume(m, 'real');
assume(g, 'real');
assume(g, 'positive');
assume(p, 'real');
assume(phi, 'real');
assume(C1, 'real');
assume(C2, 'real');
assume(C3, 'real');
assume(C4, 'real');
assume(kd, 'real');
assume(kd >= 0);

% ***********************************************
% Varying attitude, constant altitude, rate form:
% ***********************************************

g = 9.81;
X = [v phi z]';             % State Vector
f = [(g*tan(phi)); p; 0];   % State Derivative
zm = -v*(cos(phi))^2/z + p;   % State measurement

% Determine Lie derivatives:
gr = gradient(zm(1), X');
Ld1 = simplify(gr'*f);
gr = gradient(Ld1, X');
Ld2 = simplify(gr'*f);
Ld = [zm(1); Ld1; Ld2];

% For distinguishing double matches:
gr = gradient(Ld2, X');
Ld3 = simplify(gr'*f);

% Observability matrix:
O = [simplify(gradient(Ld(1), X')), ...
    simplify(gradient(Ld(2), X')), ...
    simplify(gradient(Ld(3), X'))]; %'

% Local weak observability
% Full rank means observable:
fprintf('Rank of O = %d.\n', rank(O));
if(length(X) == rank(O))
    fprintf('Observable system.\n');
else
    fprintf('Unobservable system.\n');
end

% Local observability:
s1 = solve(Ld(1)==C1, v);
Ld2_ = subs(Ld(2), v, s1);
s2 = solve(Ld2_==C2, z);
Ld3_1 = subs(Ld(3), v, s1);
Ld3_ = subs(Ld3_1, z, s2);

% assume the rate nonzero:
assume(p ~= 0);
s3 = solve(Ld3_==C3, phi, 'Real', true, 'IgnoreAnalyticConstraints', true); % , 'ReturnConditions', true
% This system has 6 solutions. Typically only one is feasible.

% Numerically verify the solutions:
threshold = 1e-5;
rng(2021);

phis_ce = [];
vs_ce = [];
zs_ce = [];

fprintf('Testing local observability:\n');

for test = 1:n_tests
    % pick a ground-truth value:
    if(METHOD == GRID)
        % get the states from a grid:
        v_0 = V(test);
        z_0 = Z(test);
        phi_0 = A(test);
        p = P(test);
    else
        % randomly generate states:
        v_0 = v_max * (2*rand(1)-1);
        z_0 = rand(1) * z_max;
        phi_0 = (2*rand(1)-1)*max_angle;
        p = max_rate * (2 * rand(1) - 1);
    end
    
    if(phi_0 == 0)
        phi_0 = 1E-3;
    end
    v = v_0; z = z_0; phi = phi_0;
    if(p == 0)
        p = 1e-5 + rand(1);
    end
    
    % verbose:
    fprintf('%d/%d: v = %f, z = %f, phi = %f, p = %f. ', test, n_tests, v, z, phi, p);
    
    % generate corresponding observations:
    C1 = subs(Ld(1));
    C2 = subs(Ld(2));
    C3 = subs(Ld(3));
    % For distinguishing between double solutions:
    C4 = subs(Ld3);
    
    % Solve for the state, given the observation vector:
    % Going over all 6 solutions:
    solved = false;
    for i = 1:6
        
        phi_sol = double(subs(s3(i)));
        phi = phi_sol;
        
        if(phi == 0 || abs(abs(phi) - 0.5*pi) < 1E-3)
            % Prevent divide by zero.
            z = -1;
            z_sol = z;
            v = 0;
        else
            z_sol = double(subs(s2));
            z = z_sol;
            v_sol = double(subs(s1));
            v = v_sol;
        end
        if(z_sol >= 0 && phi_sol >= -max_angle && phi_sol <= max_angle && imag(phi_sol) == 0 &&  imag(z_sol) == 0 && imag(v_sol) == 0)
            
            v = v_sol; phi = phi_sol; z = z_sol;
            CC4 = subs(Ld3);
            
            if(abs(double(C4) - double(CC4)) < 1e-5 && (abs(z_0-z_sol) > threshold || abs(v_0-v_sol) > threshold  || abs(phi_0-phi_sol) > threshold))
                fprintf('Found a counterexample!\n');
                fprintf('Solution %d: phi, z, v = (%f, %f, %f) = (%f, %f, %f)\n', i, (phi_sol / pi) * 180, z_sol, v_sol, (phi_0 / pi) * 180, z_0, v_0);
                fprintf('C4 = %f, CC4 = %f\n', C4, CC4);
                phis_ce = [phis_ce; rad2deg(phi_sol)];
                vs_ce = [vs_ce; v_sol];
                zs_ce = [zs_ce; z_sol];
            else
                if(~solved)
                    fprintf('Correct!\n');
                end
            end
            
            solved = true;
        end
    end
end
fprintf('\nDone!\n');

% report on the results:
n_counter_examples = length(phis_ce);
if(n_counter_examples > 0)
    fprintf('Unsuccessful: number of counterexamples: %d.\n', n_counter_examples);
else
    fprintf('Successful verification: no counterexamples.\n');
end