function DO = observability_deg_constant_alt_changing_att(X, p, g, kd, m, DIVERGENCE)
% function DO = observability_deg_constant_alt_changing_att(X, p, g, kd, m, DIVERGENCE)
%
% Function for calculating Degree of Observability at a point in state
% space.
%
% - X: 3D state space: [v, phi, z]
% - p: roll rate in rad/s
% - g: gravity (m/s^2)
% - kd: drag parameter
% - m: mass in kg
% - DIVERGENCE: whether the robot observes optic flow divergence
%
% - DO: Degree of observability
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

% The first arguments have to be provided:
if(~exist('DIVERGENCE', 'var') || isempty(DIVERGENCE))
    DIVERGENCE = false;
end

% state variables:
v = X(1);
phi = X(2);
z = X(3);

if(~exist('kd', 'var') || isempty(kd) || kd == 0)
    % No drag: 
    % Observability matrix at this point in state space:
    if(~DIVERGENCE)
        % Only ventral flow:
        O(1, :) = [-cos(phi)^2/z,       p*sin(2*phi)/z,                 2*p^2*cos(2*phi)/z];
        O(2, :) = [sin(2*phi)*v/z,      (2*p*v-g)*cos(2*phi)/z,         4*p*(g-p*v)*sin(2*phi)/z];
        O(3, :) = [cos(phi)^2*v/z^2,    (g-2*p*v)*sin(2*phi)/(2*z^2),   -p*(2*(p*v-g)*cos(2*phi)+g)/z^2];
    else
        % Also divergence:
        O(1, :) = [     -cos(phi)^2/z,  (v*sin(2*phi))/z,     (v*cos(phi)^2)/z^2];
        O(2, :) = [ -sin(2*phi)/(2*z), -(v*cos(2*phi))/z, (v*sin(2*phi))/(2*z^2)];
        O(3, :) = [                 0, -(g*cos(2*phi))/z, (g*sin(2*phi))/(2*z^2)];
    end
else
    % Drag:
    if(~DIVERGENCE)
        % Only ventral flow:
        O(1, :) = [      -cos(phi)^2/z,                       (cos(phi)*(kd*cos(phi) + 2*m*p*sin(phi)))/(m*z),                                                                 -(kd^2*cos(2*phi) + kd^2 - 4*m^2*p^2*cos(2*phi) + 4*kd*m*p*sin(2*phi))/(2*m^2*z)];
        O(2, :) = [   (v*sin(2*phi))/z,        -(g*m*cos(2*phi) + kd*v*sin(2*phi) - 2*m*p*v*cos(2*phi))/(m*z),                          (v*sin(2*phi)*kd^2 - 4*v*cos(2*phi)*kd*m*p + g*cos(2*phi)*kd*m - 4*v*sin(2*phi)*m^2*p^2 + 4*g*sin(2*phi)*m^2*p)/(m^2*z)];
        O(3, :) = [ (v*cos(phi)^2)/z^2, -(cos(phi)*(kd*v*cos(phi) - g*m*sin(phi) + 2*m*p*v*sin(phi)))/(m*z^2), (kd^2*v - 2*g*m^2*p + kd^2*v*cos(2*phi) + 4*g*m^2*p*cos(2*phi) - 4*m^2*p^2*v*cos(2*phi) - g*kd*m*sin(2*phi) + 4*kd*m*p*v*sin(2*phi))/(2*m^2*z^2)];
    else
        % Also divergence:
        O(1, :) = [         -cos(phi)^2/z,                          (v*sin(2*phi))/z,                                 (v*cos(phi)^2)/z^2];
        O(2, :) = [     -sin(2*phi)/(2*z),                         -(v*cos(2*phi))/z,                             (v*sin(2*phi))/(2*z^2)];
        O(3, :) = [ (kd*cos(phi)^2)/(m*z), -(g*m*cos(2*phi) + kd*v*sin(2*phi))/(m*z), -(cos(phi)*(kd*v*cos(phi) - g*m*sin(phi)))/(m*z^2)];
    end
end

% Get the condition number of the observability matrix and determine the
% degree of observability (DO) as described in the materials and methods:
CO = cond(O);
DO = 1/log10(CO);
