function COi = observability_deg_constant_alt_changing_att_insect(X, p, p_wings, g, kd, m, DIVERGENCE)
% function COi = observability_deg_constant_alt_changing_att_insect(X, p, p_wings, g, kd, m, DIVERGENCE)
%
% Function for calculating Degree of Observability at a point in state
% space.
%
% - X: 4D state space: [v, phi, phi_wings, z]
% - p: roll rate in rad/s
% - p_wings: roll rate of the wing stroke plane
% - g: gravity (m/s^2)
% - kd: drag parameter
% - m: mass in kg
% - DIVERGENCE: whether the robot observes optic flow divergence
%
% - COi: Degree of observability
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('DIVERGENCE', 'var') || isempty(DIVERGENCE))
    DIVERGENCE = false;
end

v = X(1);
phi = X(2);
phi_wings = X(3);
z = X(4);

if(~exist('kd', 'var') || isempty(kd) || kd == 0)
    % Observability matrix at this point in state space: CASE 17
    if(~DIVERGENCE)
        O(1,:) = [      -cos(phi)^2/z,                                                                                                   (p*sin(2*phi))/z,                                                                                                                                                                      (2*p^2*cos(2*phi))/z,  0];
        O(2,:) = [   (v*sin(2*phi))/z, (4*p*v*cos(phi)^2*cos(phi_wings) - 2*p*v*cos(phi_wings) + 2*g*cos(phi)*sin(phi)*sin(phi_wings))/(z*cos(phi_wings)), -(4*g*p*cos(phi_wings)*sin(phi_wings) - 2*g*p_wings*cos(phi)*sin(phi) - 8*g*p*cos(phi)^2*cos(phi_wings)*sin(phi_wings) + 8*p^2*v*cos(phi)*cos(phi_wings)^2*sin(phi))/(z*cos(phi_wings)^2),  1];
        O(3,:) = [                  0,                                                                   -(g*(sin(phi)^2 - 1))/(z*(sin(phi_wings)^2 - 1)),                                                                                       (2*g*cos(phi)*(2*p*cos(phi_wings)*sin(phi) - p_wings*cos(phi)*sin(phi_wings)))/(z*cos(phi_wings)^3), -1];
        O(4,:) = [ (v*cos(phi)^2)/z^2,                                                                -(p*v*sin(2*phi) - g*cos(phi)^2*tan(phi_wings))/z^2,                    (g*p_wings*cos(phi)^2 + 2*p^2*v*cos(phi_wings)^2 - 4*p^2*v*cos(phi)^2*cos(phi_wings)^2 - 4*g*p*cos(phi)*cos(phi_wings)*sin(phi)*sin(phi_wings))/(z^2*cos(phi_wings)^2),  0];

    else
        % with divergence, CASE 18
        O(1, :) = [      -cos(phi)^2/z,                                                                                                   (p*sin(2*phi))/z,  0,      -sin(2*phi)/(2*z)];
        O(2, :) = [   (v*sin(2*phi))/z, (4*p*v*cos(phi)^2*cos(phi_wings) - 2*p*v*cos(phi_wings) + 2*g*cos(phi)*sin(phi)*sin(phi_wings))/(z*cos(phi_wings)),  1,      -(v*cos(2*phi))/z];
        O(3, :) = [                  0,                                                                   -(g*(sin(phi)^2 - 1))/(z*(sin(phi_wings)^2 - 1)), -1,                      0];
        O(4, :) = [ (v*cos(phi)^2)/z^2,                                                                -(p*v*sin(2*phi) - g*cos(phi)^2*tan(phi_wings))/z^2,  0, (v*sin(2*phi))/(2*z^2)];
    end
else
    if(~exist('m', 'var') || isempty(m))
        error('Drag model requires also mass.');
    end
    
    if(~DIVERGENCE)
        O(1,:) = [      -cos(phi)^2/z,                                                     (cos(phi)*(kd*cos(phi) + 2*m*p*sin(phi)))/(m*z),                                                                                                                                                                              -(kd^2*cos(2*phi) + kd^2 - 4*m^2*p^2*cos(2*phi) + 4*kd*m*p*sin(2*phi))/(2*m^2*z),  0];
        O(2,:) = [   (v*sin(2*phi))/z, (2*cos(phi)*sin(phi)*(g*tan(phi_wings) - (kd*v)/m))/z + (2*p*v*cos(phi)^2)/z - (2*p*v*sin(phi)^2)/z, ((kd*v - g*m*tan(phi_wings))*(kd*sin(2*phi) - 2*m*p*cos(2*phi)))/(m^2*z) - (2*p*(kd*v*cos(phi)^2 - kd*v*sin(phi)^2 - g*m*cos(phi)^2*tan(phi_wings) + g*m*sin(phi)^2*tan(phi_wings) + 2*m*p*v*sin(2*phi)))/(m*z) + (g*p_wings*sin(2*phi))/(z*cos(phi_wings)^2),  1];
        O(3,:) = [                  0,                                                    -(g*(sin(phi)^2 - 1))/(z*(sin(phi_wings)^2 - 1)),                                                                                                                        (g*cos(phi)*(kd*cos(phi)*cos(phi_wings) + 4*m*p*cos(phi_wings)*sin(phi) - 2*m*p_wings*cos(phi)*sin(phi_wings)))/(m*z*cos(phi_wings)^3), -1];
        O(4,:) = [ (v*cos(phi)^2)/z^2,                      (cos(phi)^2*(g*tan(phi_wings) - (kd*v)/m))/z^2 - (2*p*v*cos(phi)*sin(phi))/z^2,     (g*p_wings*cos(phi)^2*(tan(phi_wings)^2 + 1))/z^2 - p*((2*cos(phi)*sin(phi)*(g*tan(phi_wings) - (kd*v)/m))/z^2 + (2*p*v*cos(phi)^2)/z^2 - (2*p*v*sin(phi)^2)/z^2) - (g*tan(phi_wings) - (kd*v)/m)*((2*p*cos(phi)*sin(phi))/z^2 + (kd*cos(phi)^2)/(m*z^2)),  0];
    else
        O(1,:) = [      -cos(phi)^2/z,                                                     (cos(phi)*(kd*cos(phi) + 2*m*p*sin(phi)))/(m*z),  0,      -sin(2*phi)/(2*z)];
        O(2,:) = [   (v*sin(2*phi))/z, (2*cos(phi)*sin(phi)*(g*tan(phi_wings) - (kd*v)/m))/z + (2*p*v*cos(phi)^2)/z - (2*p*v*sin(phi)^2)/z,  1,      -(v*cos(2*phi))/z];
        O(3,:) = [                  0,                                                    -(g*(sin(phi)^2 - 1))/(z*(sin(phi_wings)^2 - 1)), -1,                      0];
        O(4,:) = [ (v*cos(phi)^2)/z^2,                      (cos(phi)^2*(g*tan(phi_wings) - (kd*v)/m))/z^2 - (2*p*v*cos(phi)*sin(phi))/z^2,  0, (v*sin(2*phi))/(2*z^2)];
 
    end
end
try
    CO = cond(O);
    COi = 1/log10(CO);  %% Degree of observability for complete system
catch
    fprintf('?');
end
