function DO = observability_deg_constant_alt_changing_att_moment(X, M, I, m, kd, g, MEASURE_RATE)
% function DO = observability_deg_constant_alt_changing_att_moment(X, M, I, m, kd, g, MEASURE_RATE)
%
% Function for calculating Degree of Observability at a point in state
% space.
%
% - X: 4D state space: [v, phi, p, z]
% - M: commanded moment (kg m^2 / s^2)
% - I: moment of inertia (kg m^2)
% - g: gravity (m/s^2)
% - kd: drag parameter
% - m: mass in kg
% - MEASURE_RATE: whether the robot observes rotation rates
%
% - DO: Degree of observability
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

v = X(1);
phi = X(2);
p = X(3);
z = X(4);

if(~MEASURE_RATE)
    % Observability matrix:
    O(1, :) = [      -cos(phi)^2/z,                              (2*kd*v*cos(phi)^2 + m*p*sin(2*phi))/(m*z),                                           -(3*I*kd^2*v^2 - M*m^2*sin(2*phi) - 2*I*m^2*p^2*cos(2*phi) + 3*I*kd^2*v^2*cos(2*phi) - I*g*kd*m*sin(2*phi) + 4*I*kd*m*p*v*sin(2*phi))/(I*m^2*z),                                                                                                                                                                                                                                                                                                   (12*I*kd^3*v^3 + 6*M*m^3*p*cos(2*phi) + 12*I*kd^3*v^3*cos(2*phi) - 4*I*m^3*p^3*sin(2*phi) - 6*M*kd*m^2*v*sin(2*phi) - 4*I*g*kd*m^2*p - 8*I*g*kd^2*m*v*sin(2*phi) - 12*I*kd*m^2*p^2*v*cos(2*phi) + 18*I*kd^2*m*p*v^2*sin(2*phi) + 6*I*g*kd*m^2*p*cos(2*phi))/(I*m^3*z)];
    O(2, :) = [   (v*sin(2*phi))/z,        -(kd*sin(2*phi)*v^2 - 2*m*p*cos(2*phi)*v + g*m*cos(2*phi))/(m*z),                      (2*I*sin(2*phi)*kd^2*v^3 - 4*I*cos(2*phi)*kd*m*p*v^2 + 2*I*g*cos(2*phi)*kd*m*v - 4*I*sin(2*phi)*m^2*p^2*v + 4*I*g*sin(2*phi)*m^2*p + 2*M*cos(2*phi)*m^2*v)/(I*m^2*z), -(2*I*g*m^3*p^2 + 12*I*kd^3*v^4*cos(phi)^3*sin(phi) + 12*I*g*m^3*p^2*cos(phi)^2 - 24*I*g*m^3*p^2*cos(phi)^4 - 6*M*kd*m^2*v^2*cos(phi)^2 + 12*M*kd*m^2*v^2*cos(phi)^4 - 8*I*m^3*p^3*v*cos(phi)^2 + 16*I*m^3*p^3*v*cos(phi)^4 - 12*M*g*m^3*cos(phi)^3*sin(phi) - 8*I*g*kd^2*m*v^2*cos(phi)^2 + 16*I*g*kd^2*m*v^2*cos(phi)^4 + 12*I*kd^2*m*p*v^3*cos(phi)^2 - 24*I*kd^2*m*p*v^3*cos(phi)^4 + 24*M*m^3*p*v*cos(phi)^3*sin(phi) - 4*I*g^2*kd*m^2*cos(phi)^3*sin(phi) - 24*I*kd*m^2*p^2*v^2*cos(phi)^3*sin(phi) + 24*I*g*kd*m^2*p*v*cos(phi)^3*sin(phi))/(I*m^3*z*cos(phi)^2)];
    O(3, :) = [                  1,                                                        (v*sin(2*phi))/z,                                                                                                               (- 2*kd*sin(2*phi)*v^2 + 4*m*p*cos(2*phi)*v + g*m - 2*g*m*cos(2*phi))/(m*z),                                                                                                                                                                                                                            (12*I*kd^2*v^3*cos(phi)^2*sin(phi) + 6*M*m^2*v*cos(phi)*(2*cos(phi)^2 - 1) + 4*I*g*m^2*p*sin(phi)*(2*cos(phi)^2 - 1) + 16*I*g*m^2*p*cos(phi)^2*sin(phi) - 24*I*m^2*p^2*v*cos(phi)^2*sin(phi) + 2*I*g*kd*m*v*cos(phi)*(2*cos(phi)^2 - 1) - 8*I*g*kd*m*v*cos(phi)*sin(phi)^2 - 12*I*kd*m*p*v^2*cos(phi)*(2*cos(phi)^2 - 1))/(I*m^2*z*cos(phi))];
    O(4, :) = [ (v*cos(phi)^2)/z^2, -(cos(phi)*(kd*cos(phi)*v^2 + 2*m*p*sin(phi)*v - g*m*sin(phi)))/(m*z^2), (I*kd^2*v^3 - M*m^2*v*sin(2*phi) - I*g*m^2*p + I*kd^2*v^3*cos(2*phi) + 2*I*g*m^2*p*cos(2*phi) - 2*I*m^2*p^2*v*cos(2*phi) + 2*I*kd*m*p*v^2*sin(2*phi) - I*g*kd*m*v*sin(2*phi))/(I*m^2*z^2),                                                                                                                         - (p*(2*I*sin(2*phi)*kd^2*v^3 - 4*I*cos(2*phi)*kd*m*p*v^2 + 2*I*g*cos(2*phi)*kd*m*v - 4*I*sin(2*phi)*m^2*p^2*v + 4*I*g*sin(2*phi)*m^2*p + 2*M*cos(2*phi)*m^2*v))/(I*m^2*z^2) - (M*(- 2*kd*sin(2*phi)*v^2 + 4*m*p*cos(2*phi)*v + g*m - 2*g*m*cos(2*phi)))/(I*m*z^2) - ((kd*v^2 - g*m*tan(phi))*(3*I*kd^2*v^2 - M*m^2*sin(2*phi) - 2*I*m^2*p^2*cos(2*phi) + 3*I*kd^2*v^2*cos(2*phi) - I*g*kd*m*sin(2*phi) + 4*I*kd*m*p*v*sin(2*phi)))/(I*m^3*z^2)];
    
    % Get condition number:
    CO = cond(O);
    if(isinf(CO))
        DO = 0;
    else
        DO = 1/log10(CO);  %% Degree of observability for complete system
    end
else
    
    % Observability matrix when including ventral flow, its Lie derivatives and rate in the observation vector:
    O(1, :) = [      -cos(phi)^2/z, 0,                 (p*sin(2*phi))/z,                                     (2*I*cos(2*phi)*p^2 + M*sin(2*phi))/(I*z)];
    O(2, :) = [   (v*sin(2*phi))/z, 0,      -(cos(2*phi)*(g - 2*p*v))/z,        (- 4*I*v*sin(2*phi)*p^2 + 4*I*g*sin(2*phi)*p + 2*M*v*cos(2*phi))/(I*z)];
    O(3, :) = [                  1, 1,                 (v*sin(2*phi))/z,                                     (g - 2*g*cos(2*phi) + 4*p*v*cos(2*phi))/z];
    O(4, :) = [ (v*cos(phi)^2)/z^2, 0, (sin(2*phi)*(g - 2*p*v))/(2*z^2), -(M*v*sin(2*phi) + I*g*p + 2*I*p^2*v*cos(2*phi) - 2*I*g*p*cos(2*phi))/(I*z^2)];
    
    CO = cond(O);
    if(isinf(CO))
        COi1 = 0;
    else
        COi1 = 1/log10(CO);  %% Degree of observability for complete system
    end
    
    % Observability matrix when including just ventral flow and its Lie derivatives in the observation vector:
    O(1, :) = [      -cos(phi)^2/z,                 (p*sin(2*phi))/z,                                     (2*I*cos(2*phi)*p^2 + M*sin(2*phi))/(I*z),                                                                                                                                          (- 4*I*sin(2*phi)*p^3 + 6*M*cos(2*phi)*p)/(I*z)];
    O(2, :) = [   (v*sin(2*phi))/z,      -(cos(2*phi)*(g - 2*p*v))/z,        (- 4*I*v*sin(2*phi)*p^2 + 4*I*g*sin(2*phi)*p + 2*M*v*cos(2*phi))/(I*z), -(2*I*g*p^2 + 12*I*g*p^2*cos(phi)^2 - 24*I*g*p^2*cos(phi)^4 - 8*I*p^3*v*cos(phi)^2 + 16*I*p^3*v*cos(phi)^4 - 12*M*g*cos(phi)^3*sin(phi) + 24*M*p*v*cos(phi)^3*sin(phi))/(I*z*cos(phi)^2)];
    O(3, :) = [                  1,                 (v*sin(2*phi))/z,                                     (g - 2*g*cos(2*phi) + 4*p*v*cos(2*phi))/z,                                                  -(6*M*v*cos(phi) - 12*M*v*cos(phi)^3 + 4*I*g*p*sin(phi) + 24*I*p^2*v*cos(phi)^2*sin(phi) - 24*I*g*p*cos(phi)^2*sin(phi))/(I*z*cos(phi))];
    O(4, :) = [ (v*cos(phi)^2)/z^2, (sin(2*phi)*(g - 2*p*v))/(2*z^2), -(M*v*sin(2*phi) + I*g*p + 2*I*p^2*v*cos(2*phi) - 2*I*g*p*cos(2*phi))/(I*z^2),      (6*M*g*cos(phi)^3 - 5*M*g*cos(phi) + 6*M*p*v*cos(phi) + 2*I*g*p^2*sin(phi) - 12*M*p*v*cos(phi)^3 + 8*I*p^3*v*cos(phi)^2*sin(phi) - 12*I*g*p^2*cos(phi)^2*sin(phi))/(I*z^2*cos(phi))];

    CO = cond(O);
    if(isinf(CO))
        COi2 = 0;
    else
        COi2 = 1/log10(CO);  %% Degree of observability for complete system
    end
    
    % Since we can choose which observation vector to use, we choose the
    % one that results  in maximal observability:
    DO = max([COi1, COi2]);
end

