function plot_attitude_experiment(filename, start_time, end_time, COMPARE_HISTOGRAMS, DETERMINE_ERRORS, PLOT_THETA, PLOT_OPTITRACK, FLIP_VX, COMPARE_PHI, plot_name)
% function plot_attitude_experiment(filename, start_time, end_time, COMPARE_HISTOGRAMS, DETERMINE_ERRORS, PLOT_THETA, PLOT_OPTITRACK, FLIP_VX, COMPARE_PHI,  plot_name)
%
% - filename: filename of the Paparazzi autopilot log file
% - start_time: start time in seconds for where to start plotting
% - end_time: end time in seconds for plotting
% - COMPARE_HISTOGRAMS: whether to compare histograms of IMU vs OF
% - DETERMINE_ERRORS: whether to determine the errors
% - PLOT_THETA: most experiments only involve roll. This variable indicates
% whether we should also plot the pitch angle.
% - PLOT_OPTITRACK: Whether to plot optitrack attitude if available.
% - FLIP_VX: whether to flip vx - optitrack settings have changed over
% time...
% - COMPARE_PHI: when true, phi is used for histograms and errors, else
% theta.
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('start_time', 'var') || isempty(start_time))
    start_time = []; end_time = [];
end
if(~exist('PLOT_THETA', 'var') || isempty(PLOT_THETA))
    PLOT_THETA = false;
end
if(~exist('COMPARE_HISTOGRAMS', 'var') || isempty(COMPARE_HISTOGRAMS))
    COMPARE_HISTOGRAMS = true;
end
if(~exist('DETERMINE_ERRORS', 'var') || isempty(DETERMINE_ERRORS))
    DETERMINE_ERRORS = true;
end
if(~exist('PLOT_OPTITRACK', 'var') || isempty(PLOT_OPTITRACK))
    PLOT_OPTITRACK = false;
end
if(~exist('FLIP_VX', 'var') || isempty(FLIP_VX))
    FLIP_VX = true;
end
if(~exist('COMPARE_PHI', 'var') || isempty(COMPARE_PHI))
    COMPARE_PHI = true;
end
if(~exist('plot_name', 'var') || isempty(plot_name))
    PLOT_NAME = false;
else 
    PLOT_NAME = true;
end

compare_intervals = [];
errors_interval = [];

% get colors for plotting:
color = get_color_palette();
OF = 1;
IMU = 2;
color_OF = color.flow;
color_IMU = color.imu;
phi_color = color.phi;
theta_color = color.theta;
vy_color = color.vy;
vx_color = color.vx;
z_color = color.z;

% whether to fill the background according to which estimate is used,
% IMU-based or optic-flow-based:
FILL_BACKGROUND = true;
face_alpha = 0.15;

% read the paparazzi log:
[GPS, INS_FLOW_INFO, INS, OT_ATT] = PprzLogReader_1_0_attitude(filename);

if(size(OT_ATT,1) <= 1)
    PLOT_OPTITRACK = false;
end
% indices of the different (state) variables:
time_ind = 1;
vy_ind = 2;
phi_ind = 3;
vx_ind = 5;
theta_ind = 6;
z_ind = 8;
use_ind = 19;
phi_GT_ind = 11;
vx_GT_ind = 13;
theta_GT_ind = 14;
vy_GT_ind = 10;
z_GT_ind = 16;

if(FLIP_VX)
    INS_FLOW_INFO(:, vx_GT_ind) = -INS_FLOW_INFO(:, vx_GT_ind); 
end

optitrack_phi_ind = 2;
optitrack_theta_ind = 3;

% set start and end indices:
if(~isempty(start_time))
    start_ind = find_ind_for_time(INS_FLOW_INFO(:,time_ind), start_time);
else
    start_ind = 1;
end
if(~isempty(end_time))
    end_ind = find_ind_for_time(INS_FLOW_INFO(:,time_ind), end_time);
else
    end_ind = size(INS_FLOW_INFO, 1);
end
start_time = INS_FLOW_INFO(start_ind,time_ind);
end_time = INS_FLOW_INFO(end_ind,time_ind);
INS_FLOW_INFO = INS_FLOW_INFO(start_ind:end_ind, :);

% correct for miscalibration optitrack attitude by subtracting average (zero-centered):
OT_ATT(:,optitrack_phi_ind)  = OT_ATT(:, optitrack_phi_ind) - mean(OT_ATT(start_ind:end_ind, optitrack_phi_ind));
OT_ATT(:,optitrack_theta_ind)  = OT_ATT(:, optitrack_theta_ind) - mean(OT_ATT(start_ind:end_ind, optitrack_theta_ind));

% font and plot styles:
fs = 12; gt_style = '--';

% if the pitch was also estimated, plot it together with the roll angle:
if(PLOT_THETA)
    ALL_PLOTS = true;
    
    
    if(PLOT_NAME)
        figure('Name', [plot_name ' - pitch and roll'], 'NumberTitle', 'off');
    else
        figure();
    end
    set(gcf, 'Color', [1 1 1]);
    
    % PHI
    if(~ALL_PLOTS)
        subplot(2,1,1);
    else
        subplot(5,1,1);
    end
    hold on;
    if(FILL_BACKGROUND)
        border = 1;
        max_phi1 = max(INS_FLOW_INFO(:,phi_ind));
        max_phi2 = max(INS_FLOW_INFO(:,phi_GT_ind));
        max_phi = max([max_phi1, max_phi2]);
        
        min_phi1 = min(INS_FLOW_INFO(:,phi_ind));
        min_phi2 = min(INS_FLOW_INFO(:,phi_GT_ind));
        min_phi = min([min_phi1, min_phi2]);
        
        % find the moments where the optic-flow-based estimate was switched
        % on (starting to use it).
        inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
        inds_switch_on = inds_switch_on(1:2:end);
        
        % Only fill the background when there is one such a switching
        % moment:
        if(length(inds_switch_on) == 1)
            
            xx = [INS_FLOW_INFO(1,time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind),  INS_FLOW_INFO(1,time_ind)];
            yy = [max_phi+border, max_phi+border, min_phi-border, min_phi-border];
            hh = fill(xx, yy, color.imu,'LineStyle','none');
            hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
            set(hh, 'facealpha', face_alpha);
            
            xx = [INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(end,time_ind), INS_FLOW_INFO(end,time_ind),  INS_FLOW_INFO(inds_switch_on(1),time_ind)];
            yy = [max_phi+border, max_phi+border, min_phi-border, min_phi-border];
            hh = fill(xx, yy, color.flow,'LineStyle','none');
            hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
            set(hh, 'facealpha', face_alpha);
        end
    end
    % plot the estimated roll angle, ground-truth roll angle, and 0.
    plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,phi_ind), 'Color', phi_color);
    plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,phi_GT_ind), gt_style, 'Color', phi_color);
    if(PLOT_OPTITRACK)
        plot(OT_ATT(:, time_ind), OT_ATT(:, optitrack_phi_ind), gt_style, 'Color', color.optitrack);
    end
    plot([INS_FLOW_INFO(1, time_ind) INS_FLOW_INFO(end, time_ind)], [0, 0], '--', 'Color', [0 0 0], 'LineWidth', 1)

    if(exist('start_time', 'var'))
        if(FILL_BACKGROUND)
            ylim([min_phi-border, max_phi + border]);
        end
        xlim([start_time, end_time]);
    end
    ylabel('\phi [\circ]');
    set(gca,'FontSize', fs);
    
    %THETA
    if(~ALL_PLOTS)
        subplot(2,1,2);
    else
        subplot(5,1,2);
    end
    hold on;
    if(FILL_BACKGROUND)
        border = 1;
        max_theta1 = max(INS_FLOW_INFO(:,theta_ind));
        max_theta2 = max(INS_FLOW_INFO(:,theta_GT_ind));
        max_theta = max([max_theta1, max_theta2]);
        
        min_theta1 = min(INS_FLOW_INFO(:,theta_ind));
        min_theta2 = min(INS_FLOW_INFO(:,theta_GT_ind));
        min_theta = min([min_theta1, min_theta2]);
        
        % find the moments where the optic-flow-based estimate was switched
        % on (starting to use it).
        inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
        inds_switch_on = inds_switch_on(1:2:end);
        
        % Only fill the background when there is one such a switching
        % moment:
        if(length(inds_switch_on) == 1)
            
            xx = [INS_FLOW_INFO(1,time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind),  INS_FLOW_INFO(1,time_ind)];
            yy = [max_theta+border, max_theta+border, min_theta-border, min_theta-border];
            hh = fill(xx, yy, color.imu,'LineStyle','none');
            hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
            set(hh, 'facealpha', face_alpha);
            
            xx = [INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(end,time_ind), INS_FLOW_INFO(end,time_ind),  INS_FLOW_INFO(inds_switch_on(1),time_ind)];
            yy = [max_theta+border, max_theta+border, min_theta-border, min_theta-border];
            hh = fill(xx, yy, color.flow,'LineStyle','none');
            hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
            set(hh, 'facealpha', face_alpha);
        end
    end
    % plot the estimated pitch angle, ground-truth pitch angle, and 0.
    plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,theta_ind), 'Color', theta_color);
    plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,theta_GT_ind), gt_style, 'Color', theta_color);
    if(PLOT_OPTITRACK)
        plot(OT_ATT(:, time_ind), OT_ATT(:, optitrack_theta_ind), gt_style, 'Color', color.optitrack);
    end
    plot([INS_FLOW_INFO(1, time_ind) INS_FLOW_INFO(end, time_ind)], [0, 0], '--', 'Color', [0 0 0], 'LineWidth', 1)
    if(exist('start_time', 'var'))
        if(FILL_BACKGROUND)
            ylim([min_theta-border, max_theta + border]);
        end
        xlim([start_time, end_time]);
    end
    ylabel('\theta [\circ]');
    set(gca,'FontSize', fs);
    
    if(ALL_PLOTS)
        % VELOCITY:
        subplot(5,1,3);
        hold on;
        if(FILL_BACKGROUND)
            border = 0.10;
            max_vy1 = max(INS_FLOW_INFO(:,vy_ind));
            max_vy2 = max(INS_FLOW_INFO(:,vy_GT_ind));
            max_vy = max([max_vy1, max_vy2]);
            
            min_vy1 = min(INS_FLOW_INFO(:,vy_ind));
            min_vy2 = min(INS_FLOW_INFO(:,vy_GT_ind));
            min_vy = min([min_vy1, min_vy2]);
            
            inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
            inds_switch_on = inds_switch_on(1:2:end);
            if(length(inds_switch_on) == 1)
                
                xx = [INS_FLOW_INFO(1,time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind),  INS_FLOW_INFO(1,time_ind)];
                yy = [max_vy+border, max_vy+border, min_vy-border, min_vy-border];
                hh = fill(xx, yy, color.imu,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
                
                xx = [INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(end,time_ind), INS_FLOW_INFO(end,time_ind),  INS_FLOW_INFO(inds_switch_on(1),time_ind)];
                yy = [max_vy+border, max_vy+border, min_vy-border, min_vy-border];
                hh = fill(xx, yy, color.flow,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
            end
        end
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,vy_ind), 'Color', vy_color);
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,vy_GT_ind), gt_style, 'Color', vy_color);
        plot([INS_FLOW_INFO(1, time_ind) INS_FLOW_INFO(end, time_ind)], [0, 0], '--', 'Color', [0 0 0], 'LineWidth', 1)
        if(exist('start_time', 'var'))
            if(FILL_BACKGROUND)
                ylim([min_vy - border, max_vy + border]);
            end
            xlim([start_time, end_time]);
        end
        ylabel('v_y [m/s]');
        set(gca,'FontSize', fs);
        
        subplot(5,1,4);
        hold on;
        if(FILL_BACKGROUND)
            border = 0.10;
            max_vx1 = max(INS_FLOW_INFO(:,vx_ind));
            max_vx2 = max(INS_FLOW_INFO(:,vx_GT_ind));
            max_vx = max([max_vx1, max_vx2]);
            
            min_vx1 = min(INS_FLOW_INFO(:,vx_ind));
            min_vx2 = min(INS_FLOW_INFO(:,vx_GT_ind));
            min_vx = min([min_vx1, min_vx2]);
            
            inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
            inds_switch_on = inds_switch_on(1:2:end);
            if(length(inds_switch_on) == 1)
                
                xx = [INS_FLOW_INFO(1,time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind),  INS_FLOW_INFO(1,time_ind)];
                yy = [max_vx+border, max_vx+border, min_vx-border, min_vx-border];
                hh = fill(xx, yy, color.imu,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
                
                xx = [INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(end,time_ind), INS_FLOW_INFO(end,time_ind),  INS_FLOW_INFO(inds_switch_on(1),time_ind)];
                yy = [max_vx+border, max_vx+border, min_vx-border, min_vx-border];
                hh = fill(xx, yy, color.flow,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
            end
        end
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,vx_ind), 'Color', vx_color);
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,vx_GT_ind), gt_style, 'Color', vx_color);
        plot([INS_FLOW_INFO(1, time_ind) INS_FLOW_INFO(end, time_ind)], [0, 0], '--', 'Color', [0 0 0], 'LineWidth', 1)
        if(exist('start_time', 'var'))
            if(FILL_BACKGROUND)
                ylim([min_vx - border, max_vx + border]);
            end
            xlim([start_time, end_time]);
        end
        ylabel('v_x [m/s]');
        set(gca,'FontSize', fs);
        
        % HEIGHT:
        subplot(5,1,5);
        hold on;
        if(FILL_BACKGROUND)
            border = 0.5;
            max_z1 = max(INS_FLOW_INFO(:,z_ind));
            max_z2 = max(INS_FLOW_INFO(:,z_GT_ind));
            max_z = max([max_z1, max_z2]);
            
            min_z1 = min(INS_FLOW_INFO(:,z_ind));
            min_z2 = min(INS_FLOW_INFO(:,z_GT_ind));
            min_z = min([min_z1, min_z2]);
            
            inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
            inds_switch_on = inds_switch_on(1:2:end);
            if(length(inds_switch_on) == 1)
                
                xx = [INS_FLOW_INFO(1,time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(inds_switch_on(1),time_ind),  INS_FLOW_INFO(1,time_ind)];
                yy = [max_z+border, max_z+border, min_z-border, min_z-border];
                hh = fill(xx, yy, color.imu,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
                
                xx = [INS_FLOW_INFO(inds_switch_on(1),time_ind), INS_FLOW_INFO(end,time_ind), INS_FLOW_INFO(end,time_ind),  INS_FLOW_INFO(inds_switch_on(1),time_ind)];
                yy = [max_z+border, max_z+border, min_z-border, min_z-border];
                hh = fill(xx, yy, color.flow,'LineStyle','none');
                hh.Annotation.LegendInformation.IconDisplayStyle = 'off';
                set(hh, 'facealpha', face_alpha);
            end
        end
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,z_ind), 'Color', z_color);
        plot(INS_FLOW_INFO(:,time_ind), INS_FLOW_INFO(:,z_GT_ind), gt_style, 'Color', z_color);
        if(exist('start_time', 'var'))
            xlim([start_time, end_time]);
            if(FILL_BACKGROUND)
                ylim([min_z-border, max_z + border]);
            end
        end
        ylabel('z [m]');
        set(gca,'FontSize', fs);
    end
    
    xlabel('Time [s]');
end


% ************************ %
%        STATISTICS
% ************************ %


if(COMPARE_PHI)
    angle_ind = phi_ind;
    angle_GT_ind = phi_GT_ind;
    v_ind = vy_ind;
    v_GT_ind = vy_GT_ind;
    label_angle = '\phi [\circ]';
    label_v = '';
else
    angle_ind = theta_ind;
    angle_GT_ind = theta_GT_ind;
    v_ind = vx_ind;
    v_GT_ind = vx_GT_ind;
    label_angle = '\theta [\circ]';
end

% PLOT WITH HISTOGRAM FOR IMU AND OPTIC-FLOW-BASED ESTIMATES:
if(COMPARE_HISTOGRAMS)
    if(isempty(compare_intervals))
        
        % whether to show the ground-truth roll angle in the histograms:
        ANGLE_GT = true;
        
        % automatic detection:
        if(PLOT_NAME)
            figure('Name', [plot_name ' - histograms'], 'NumberTitle', 'off');
        else
            figure();
        end
        set(gcf, 'Color', [1 1 1]);
        hold on;
        % Histogram when using optic-flow-based estimate
        inds_OF = find(INS_FLOW_INFO(:, use_ind) >= 0.5);
        fc = color_OF;
        if(ANGLE_GT)
            histogram(INS_FLOW_INFO(inds_OF, angle_GT_ind), 30, 'Normalization', 'probability', 'FaceAlpha', 0.5, 'BinWidth', 0.1, 'FaceColor', fc);
            if(COMPARE_PHI)
                fprintf('std(phi_OF) = %f.\n', std(INS_FLOW_INFO(inds_OF,angle_GT_ind)));
            else
                fprintf('std(theta_OF) = %f.\n', std(INS_FLOW_INFO(inds_OF,angle_GT_ind)));
            end
        else
            histogram(INS_FLOW_INFO(inds_OF,angle_ind), 30, 'Normalization', 'probability', 'FaceAlpha', 0.5, 'BinWidth', 0.1, 'FaceColor', fc);
            if(COMPARE_PHI)
                fprintf('std(phi_OF) = %f.\n', std(INS_FLOW_INFO(inds_OF,angle_ind)));
            else
                fprintf('std(theta_OF) = %f.\n', std(INS_FLOW_INFO(inds_OF,angle_ind)));
            end
        end
        
        % This contains also manual flight. Therefore, we take the last 5
        % seconds before the use of the attitude estimate starts.
        inds_switch_on = find(gradient(INS_FLOW_INFO(:, use_ind)) > 0.2);
        inds_switch_on = inds_switch_on(1:2:end);
        fprintf('Number of flights: %d\n', length(inds_switch_on));
        inds_IMU = [];
        for i = 1:length(inds_switch_on)
            extra_inds = (inds_switch_on(i)-100:inds_switch_on(i))';
            inds_IMU = [inds_IMU; extra_inds];
        end
        % Plot the histogram for the IMU-based attitudes:
        fc = color_IMU;
        if(ANGLE_GT)
            histogram(INS_FLOW_INFO(inds_IMU,angle_GT_ind), 30, 'Normalization', 'probability', 'FaceAlpha', 0.5, 'BinWidth', 0.1, 'FaceColor', fc);
            if(COMPARE_PHI)
                fprintf('std(phi_IMU) = %f.\n', std(INS_FLOW_INFO(inds_IMU,angle_GT_ind)));
            else
                fprintf('std(theta_IMU) = %f.\n', std(INS_FLOW_INFO(inds_IMU,angle_GT_ind)));
            end
        else
            histogram(INS_FLOW_INFO(inds_IMU,angle_ind), 30, 'Normalization', 'probability', 'FaceAlpha', 0.5, 'BinWidth', 0.1, 'FaceColor', fc);
            if(COMPARE_PHI)
                fprintf('std(phi_IMU) = %f.\n', std(INS_FLOW_INFO(inds_IMU,angle_ind)));
            else
                fprintf('std(theta_IMU) = %f.\n', std(INS_FLOW_INFO(inds_IMU,angle_ind)));
            end
            end
        legend('OF', 'IMU');
        xlim([-4, 4]);
        xlabel(label_angle);
        ylabel('Empirical probability [-]');
        set(gca,'FontSize', fs);
        
        % Bootstrap test on the histograms:
        WITH_RESAMPLING = true; RETAIN_LENGTHS = true; n_iterations = 10000;
        fprintf('Bootstrap with resampling:\n');
        if(ANGLE_GT)
            [p_single, p_double] = bootstrap_test(INS_FLOW_INFO(inds_IMU,angle_GT_ind), INS_FLOW_INFO(inds_OF,angle_GT_ind), n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS);
        else
            [p_single, p_double] = bootstrap_test(INS_FLOW_INFO(inds_IMU,angle_ind), INS_FLOW_INFO(inds_OF,angle_ind), n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS);
        end
        WITH_RESAMPLING = false;
        fprintf('Bootstrap without resampling:\n');
        if(ANGLE_GT)
            [p_single, p_double] = bootstrap_test(INS_FLOW_INFO(inds_IMU,angle_GT_ind), INS_FLOW_INFO(inds_OF,angle_GT_ind), n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS);
        else
            [p_single, p_double] = bootstrap_test(INS_FLOW_INFO(inds_IMU,angle_ind), INS_FLOW_INFO(inds_OF,angle_ind), n_iterations, WITH_RESAMPLING, RETAIN_LENGTHS);
        end
        
    end
end

if(DETERMINE_ERRORS)
    % automatically determine inds during which OF is in control:
    inds_OF = find(INS_FLOW_INFO(:, use_ind) >= 0.5);
    
    % calculate errors:
    z_errs = INS_FLOW_INFO(inds_OF,z_ind)-INS_FLOW_INFO(inds_OF,z_GT_ind);
    v_errs = INS_FLOW_INFO(inds_OF,v_ind)-INS_FLOW_INFO(inds_OF,v_GT_ind);
    corr_phi = INS_FLOW_INFO(inds_OF,angle_GT_ind) - mean(INS_FLOW_INFO(inds_OF,angle_GT_ind));
    angle_errs = INS_FLOW_INFO(inds_OF,angle_ind)-corr_phi;
    
    % make boxplots:
    if(PLOT_NAME)
        figure('Name', [plot_name ' - errors'], 'NumberTitle', 'off');
    else
        figure();
    end
    set(gcf, 'Color', [1 1 1]);
    hold on;
    plot([0.75, 3.25], [0 0], 'k--')
    boxplot([angle_errs, v_errs, z_errs],'PlotStyle','compact', 'Colors',[phi_color;vy_color;z_color],'OutlierSize', 1, 'Symbol', '.'); % 'BoxStyle','filled',
    xticks([1 2 3]);
    if(COMPARE_PHI)
        xticklabels({'e_{\phi}','e_{vy}','e_z'});
    else
        xticklabels({'e_{\theta}','e_{vx}','e_z'});
    end
    ylabel('Error [m], [m/s], [\circ]');
    set(gca,'FontSize', 16);
    
    % make a violinplot:
    % In the article we used the function from:
    % https://nl.mathworks.com/matlabcentral/fileexchange/45134-violin-plot
    %
    % To get the exact same plot, please download the above and uncommment the following lines:
    %
    %         figure();
    %         set(gcf, 'Color', [1 1 1]);
    %         hold on;
    %         if(COMPARE_PHI)
    %             violins = violin([angle_errs, v_errs, z_errs], 'xlabel', {'e_{\phi}','e_{vy}','e_z'}, ...
    %                 'facecolor',[color.phi; color.vy; color.z]);
    %         else
    %             violins = violin([angle_errs, v_errs, z_errs], 'xlabel', {'e_{\theta}','e_{vx}','e_z'}, ...
    %                 'facecolor',[color.phi; color.vy; color.z]);
    %         end
    %         ylabel('Error [m], [m/s], [\circ]');
    %         set(gca,'FontSize', 16);
    
    fprintf('MAE angle = %f, v = %f, z = %f.\n', mean(abs(angle_errs)), mean(abs(v_errs)), mean(abs(z_errs)));
end