function plot_flight_flapper(file_name, interval)
% function plot_flight_flapper(file_name, interval)
%
% Script for plotting the results from a flapper experiment.
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('interval', 'var') || isempty(interval))
    interval = [];
end
if(~exist('file_name', 'var') || isempty(file_name))
    file_name = '../flight_data/Flapper_log_files/log02_in_control.csv';
    interval = [425, 430];
end

% get the plotting colors:
color = get_color_palette();
gyro_color = color.gyro;
flow_color = color.flow;
phi_color = color.phi;

% MODE: 0 is not running, 1 is running, 3 is running and used.

% read the log from the micro SD card:
log = readtable(file_name);
time = log.timestamp / 1000;

%  get the start and end indices for the given time interval:
if(~isempty(interval))
    i = find(time >= interval(1));
    start_ind = i(1);
    i = find(time <= interval(2));
    end_ind = i(end);
    log = log(start_ind:end_ind, :);
    time = time(start_ind:end_ind);
end

n_time_steps = length(time);
inds = cumsum(ones(n_time_steps, 1));

% Get data from the log:
OF_time = time; 
fps = 1 ./ (OF_time(2:end) - OF_time(1:end-1));
fps(end+1) = fps(end); 
gyro_p = deg2rad(log.gyro_x);
rate_p = gyro_p;
gyro_q = deg2rad(log.gyro_y);
rate_q = gyro_q;
gyro_r = deg2rad(log.gyro_z);
rate_r = gyro_r;
att_phi = deg2rad(log.stabilizer_roll); 
att_theta = deg2rad(log.stabilizer_pitch);
att_psi = deg2rad(log.stabilizer_yaw);
cmd_thrust = log.stabilizer_thrust;
cmd_roll = log.stabilizer_roll;
flow_x = -(log.curvace_y1+log.curvace_y2+log.curvace_y3+log.curvace_y4)/4;
flow_y = (log.curvace_x1+log.curvace_x2+log.curvace_x3+log.curvace_x4)/4;
estimated_roll = log.flowest_ROLL;

% Plot for the article:
figure('Name', 'Flapper', 'NumberTitle', 'off'); 
font_info = get_font_info();
fn = font_info.name;
fs = 12; 
gt_style = '--';
set(gcf, 'Color', [1 1 1]);
subplot(3,1,1);
plot(time, rad2deg(estimated_roll), 'Color', phi_color);
hold on;
plot(time, cmd_roll, gt_style, 'Color', phi_color);
plot([time(1), time(end)], [0,0], 'k--');
legend('\phi_{OF}', '\phi_{IMU}');
ylabel('\phi [\circ]');
set(gca,'FontSize', fs, 'FontName', fn);
subplot(3,1,2);
plot(time, gyro_p, 'Color', gyro_color);
hold on
plot([time(1), time(end)], [0,0], 'k--');
ylabel('p [\circ / s]');
set(gca,'FontSize', fs, 'FontName', fn);
subplot(3,1,3);
plot(time, flow_x, 'Color', flow_color);
hold on
plot([time(1), time(end)], [0,0], 'k--');
xlabel('Time [s]');
ylabel('u [pix / s]');
set(gca,'FontSize', fs, 'FontName', fn);


