function plot_observability_vs_states(MOMENT_MODEL, MEASURE_RATE, DIVERGENCE, DRAG, PLOT_TRAJECTORY, TITLE)
% function plot_observability_vs_states(MOMENT_MODEL, MEASURE_RATE, DIVERGENCE, DRAG, PLOT_TRAJECTORY, TITLE)
%
% - MOMENT_MODEL: 4 states, including the rotation rate. Actions are
% moments.
% - MEASURE_RATE: whether the rate is measured.
% - DIVERGENCE: whether optic flow divergence is observed.
% - DRAG: whether drag is included in the model.
% - PLOT_TRAJECTORY: whether to plot a trajectory from simulation on top of the plot.
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('MOMENT_MODEL', 'var') || isempty(MOMENT_MODEL))
    % If the first argument is not given, we assume that the others are not
    % either:
    MOMENT_MODEL = true;
    MEASURE_RATE = true;
    DIVERGENCE = false;
    DRAG = false;
    PLOT_TRAJECTORY = true;
end

% scaling of the degree of observability color bar:
SCALE = true;
DO_max = 1;

% drag parameter:
if(DRAG)
    kd = 0.5;
else
    kd = 0;
end

% **************
% P vs PHI
% **************

% constant alt model:
g = 9.81;
m = 1;
v = 0;
z = 1;
M = 0; % relevant for moment model
I = 0.001; % relevant for moment model

if(PLOT_TRAJECTORY)
    
    % Determine the grid:
    p = -5:0.05:5;
    phi = -30:0.1:30;
    [PHI,P] = meshgrid(phi, p);
    
    % Determine observability degree at each grid point:
    for i = 1:1:size(p,2)
        for k = 1:1:size(phi,2)
            if(~MOMENT_MODEL)
                X = [v; phi(k)*pi/180; z];
                if(DRAG)
                    O(i, k) = observability_deg_constant_alt_changing_att(X, p(i)*pi/180, g, kd, m, DIVERGENCE);
                else
                    O(i, k) = observability_deg_constant_alt_changing_att(X, p(i)*pi/180, g, 0, m, DIVERGENCE);
                end
            else
                X = [v; phi(k)*pi/180; p(i)*pi/180; z];
                O(i, k) = observability_deg_constant_alt_changing_att_moment(X, M, I, m, kd, g, MEASURE_RATE);
            end
        end
    end
    
    % create the figure:
    figure
    contour(PHI,P,O, 'fill', 'on');
    if(PLOT_TRAJECTORY)
        hold on;
        if(MEASURE_RATE)
            load('results_hover_rate.mat');
        else
            load('results_hover.mat');
        end
        n_time_steps = size(results.state, 2);
        ind = round(0.75*n_time_steps);
        plot(smooth(rad2deg(results.state(2, ind:end))), smooth(rad2deg(results.rate(ind:end))), 'k');
    end
    xlabel('$$\phi (^o)$$', 'Interpreter', 'latex');
    ylabel('$$ p (^o/s)$$', 'Interpreter', 'latex');
    hcb1 = colorbar;
    if(SCALE)
        caxis([0, DO_max]);
    end
    set(hcb1, 'ylim', [0 DO_max])
    set(gca, 'fontsize', 20);
    set(gcf,'renderer','Painters')
    set(gcf, 'color', 'w')
    if(exist('TITLE', 'var') && ~isempty(TITLE))
        title(TITLE)
    end
    
else
    
    % **************
    % P vs V
    % **************
    
    % constant alt model:
    v = -2:0.05:2;
    z = 1;
    p = -10:0.1:10;
    phi = 5;
    [V,P] = meshgrid(v, p);
    
    % Determine observability degree at each grid point:
    for i = 1:1:size(p,2)
        for k = 1:1:size(v,2)
            if(~MOMENT_MODEL)
                X = [v(k); phi*pi/180; z];
                if(DRAG)
                    Obs(i, k) = observability_deg_constant_alt_changing_att(X, p(i)*pi/180, g, kd, m, DIVERGENCE);
                else
                    Obs(i, k) = observability_deg_constant_alt_changing_att(X, p(i)*pi/180, g, 0, m, DIVERGENCE);
                end
            else
                X = [v(k); phi*pi/180; p(i)*pi/180; z];
                Obs(i, k) = observability_deg_constant_alt_changing_att_moment(X, M, I, m, kd, g, MEASURE_RATE);
            end
        end
    end
    
    figure
    contour(V,P,Obs, 'fill', 'on');
    xlabel('$$v (m/s)$$', 'Interpreter', 'latex');
    ylabel('$$ p (^o/s)$$', 'Interpreter', 'latex');
    hcb1 = colorbar;
    if(SCALE)
        caxis([0, DO_max]);
    end
    set(hcb1, 'ylim', [0 DO_max])
    set(gca, 'fontsize', 20);
    set(gcf,'renderer','Painters')
    set(gcf, 'color', 'w')
    
    if(exist('TITLE', 'var') && ~isempty(TITLE))
        title(TITLE)
    end
end