function plot_observability_vs_states_insect()
% function plot_observability_vs_states_insect()
%
% Plot the observability degree as a function of the state.
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

SCALE = true;

% Fig 1:
FULL_MODEL = false;
MOMENT_MODEL = false;
MEASURE_RATE = true; % false for the top plot, true for bottom one
DIVERGENCE = false; % true
DRAG = true;
PLOT_TRAJECTORY = false;

% used for wings and head when not varied
fixed_phi = 0; 

DO_max = 1;

if(DRAG)
    kd = 0.5;
else
    kd = 0;
end


% **************
% P vs PHI
% **************

% constant alt model:
g = 9.81;
v = 1;
z = 1;
M = 0; % relevant for moment model
p = -5:0.05:5;
phi = -40:0.1:40;
[PHI,P] = meshgrid(phi, p);

% full model:
m = 1;
T = m*g; % this is only equal to gravity at 0 phi
I = 0.001;
w = 0;

% hover position wings
phi_wings = fixed_phi;
p_wings = 0;

for i = 1:1:size(p,2)
    for k = 1:1:size(phi,2)
        X = [v; deg2rad(phi(k)); deg2rad(phi_wings); z];
        O(i, k) = observability_deg_constant_alt_changing_att_insect(X, p(i), p_wings, g, kd, m, DIVERGENCE);
    end
end

figure
contour(PHI,P,O, 'fill', 'on');
if(PLOT_TRAJECTORY)
    hold on;
    if(MEASURE_RATE)
        load('results_hover_rate.mat');
    else
        load('results_hover.mat');
    end
    n_time_steps = size(results.state, 2);
    ind = round(0.75*n_time_steps);
    plot(smooth(rad2deg(results.state(2, ind:end))), smooth(rad2deg(results.rate(ind:end))), 'k');
end 
xlabel('$$\phi (^o)$$', 'Interpreter', 'latex');
ylabel('$$ p (^o/s)$$', 'Interpreter', 'latex');
hcb1 = colorbar;
if(SCALE)
    caxis([0, DO_max]);
end
set(hcb1, 'ylim', [0 DO_max])
set(gca, 'fontsize', 20);
set(gcf,'renderer','Painters')
set(gcf, 'color', 'w')


% **************
% P_WINGS vs PHI
% **************

% ROTATING THORAX:
p_wings = -5:0.05:5;
phi_wings = -80:0.1:80;
[PHIW,PW] = meshgrid(phi_wings, p_wings);

% head steady:
p = 0;
phi = fixed_phi;

clear O;

for i = 1:1:size(p_wings,2)
    for k = 1:1:size(phi_wings,2)
        X = [v; deg2rad(phi); deg2rad(phi_wings(k)); z];
        O(i, k) = observability_deg_constant_alt_changing_att_insect(X, p, p_wings(i), g, kd, m, DIVERGENCE);
    end
end

figure
contour(PHIW,PW,O, 'fill', 'on');
if(PLOT_TRAJECTORY)
    hold on;
    if(MEASURE_RATE)
        load('results_hover_rate.mat');
    else
        load('results_hover.mat');
    end
    n_time_steps = size(results.state, 2);
    ind = round(0.75*n_time_steps);
    plot(smooth(rad2deg(results.state(2, ind:end))), smooth(rad2deg(results.rate(ind:end))), 'k');
end 
xlabel('$$\phi_{\mathrm{wings}} (^o)$$', 'Interpreter', 'latex');
ylabel('$$ p_{\mathrm{wings}} (^o/s)$$', 'Interpreter', 'latex');
hcb1 = colorbar;
if(SCALE)
    caxis([0, DO_max]);
end
set(hcb1, 'ylim', [0 DO_max])
set(gca, 'fontsize', 20);
set(gcf,'renderer','Painters');
set(gcf, 'color', 'w');

% **************
% P vs V
% **************

% constant alt model:
g = 9.81;
z = 1;
phi = 0;
p = 0;
phi_wings = 5;
v = -2:0.05:2;
p_wings = -5:0.05:5;
[V,PW] = meshgrid(v, p_wings);

% full model:
m = 1;
M = 0;
T = m*g;
I = 0.001;
w = 0;

for i = 1:1:size(p_wings,2)
    for k = 1:1:size(v,2)
        X = [v(k); deg2rad(phi); deg2rad(phi_wings); z];
        Obs(i, k) = observability_deg_constant_alt_changing_att_insect(X, p, p_wings(i), g, kd, m, DIVERGENCE);
    end
end

figure
contour(V,PW,Obs, 'fill', 'on');
xlabel('$$ v (m/s)$$', 'Interpreter', 'latex');
ylabel('$$ p_{\mathrm{wings}} (^o/s)$$', 'Interpreter', 'latex');
hcb1 = colorbar;
if(SCALE)
    caxis([0, DO_max]);
end
set(hcb1, 'ylim', [0 DO_max])
set(gca, 'fontsize', 20);
set(gcf,'renderer','Painters')
set(gcf, 'color', 'w')

% **************
% V vs PHI WINGS
% **************

% constant alt model:
g = 9.81;
z = 1;
p = 0;
phi = 0;
p_wings = (3/180)*pi;

v = -5:0.05:5;
phi_wings = -45:1:45;
[V,PHIW] = meshgrid(v, phi_wings);

% full model:
m = 1;
M = 0;
T = m*g;
I = 0.001;
w = 0;

clear Obs
for i = 1:1:size(phi_wings,2)
    for k = 1:1:size(v,2)
        X = [v(k); deg2rad(phi); deg2rad(phi_wings(i)); z];
        Obs(i, k) = observability_deg_constant_alt_changing_att_insect(X, p, p_wings, g, kd, m, DIVERGENCE);
    end
end

figure
contour(V,PHIW, Obs, 'fill', 'on');
xlabel('$$v (m/s)$$', 'Interpreter', 'latex');
ylabel('$$\phi_{\mathrm{wings}} (^o)$$', 'Interpreter', 'latex');
hcb1 = colorbar;
if(SCALE)
    caxis([0, DO_max]);
end
set(hcb1, 'ylim', [0 DO_max])
set(gca, 'fontsize', 20);
set(gcf,'renderer','Painters')
set(gcf, 'color', 'w')





