function plots_hypothesis_faster_flight(results, results_GT)
% function plots_hypothesis_faster_flight(results, results_GT)
%
% Makes the plots for verifying the hypothesis that at higher flight
% speeds, the attitude oscillates less.
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('results', 'var') || isempty(results))
    load('results_speed_hypothesis.mat');
end
if(~exist('results_GT', 'var') || isempty(results_GT))
    plot_GT = false; 
else
    plot_GT = true;
end

% get color palette for the article:
color = get_color_palette();
lw = 2;
fs = 12; 
font_name = 'Times';
n_refs = length(results.references);

figure();
set(gcf, 'Color', [1 1 1]);
plot(results.references, results.STD_PHIs_HEAD, 'LineWidth', lw, 'Color', color.std_phi_robot);
if(plot_GT)
    hold on;
    plot(results_GT.references, results_GT.STD_PHIs_HEAD, '--', 'LineWidth', lw, 'Color', color.std_phi_robot);
end
xlabel('Reference velocities [m/s] at z = 1 m.');
ylabel('Standard deviation head attitude [\circ]');
set(findall(gcf,'-property','FontSize'),'FontSize',fs, 'FontName', font_name)

MAEs = flatten_cell(results.MAE_ESTIMATION, 2);

fig = figure();
set(gcf, 'Color', [1 1 1]);
set(fig,'defaultAxesColorOrder',[color.z; color.phi_head]);
yyaxis left;
plot(results.references, MAEs(1,:), ':', 'LineWidth', lw, 'Color', color.vy);
hold on;
plot(results.references, MAEs(4,:), '--','LineWidth', lw, 'Color', color.z);
ylabel('Mean absolute error [m, m/s]');
yyaxis right;
plot(results.references, MAEs(2,:), '-','LineWidth', lw, 'Color', color.phi_head);
hold on;
plot(results.references, MAEs(3,:), '-.','LineWidth', lw, 'Color', color.phi);
ylabel('Mean absolute attitude error [\circ]');
xlabel('Reference velocities [m/s] at z = 1 m.');
legend('v_x','z','\theta_{head}', '\theta_{body}')
set(findall(gcf,'-property','FontSize'),'FontSize',fs, 'FontName', font_name)

std_phis_no_outliers = zeros(n_refs, 1);
for r = 1:n_refs
    fPHIs = flatten_cell(results.PHIs{r});
    fPHIs = rmoutliers(fPHIs);
    std_phis_no_outliers(r) = std(fPHIs);

end

% get the bee data:
bee_data = get_bee_data();

% show the simulated data and bee data in a single plot:
fig = figure();
set(gcf, 'Color', [1 1 1]);
set(fig,'defaultAxesColorOrder',[color.std_phi_robot; color.std_phi_bee]);
yyaxis left;
plot(results.references, std_phis_no_outliers, 'LineWidth', lw, 'Color', color.std_phi_robot);
if(plot_GT)
    hold on
    plot(results.references, results_GT.STD_PHIs, '--', 'LineWidth', lw, 'Color', color.std_phi_robot);
end
ylabel('Standard deviation body attitude simulation [\circ]');
yyaxis right;
plot(bee_data.references, bee_data.std_theta, 'LineWidth', lw, 'Color', color.std_theta_bee);
ylabel('Standard deviation body attitude honeybees [\circ]');
xlabel('Velocities [m/s].');
set(gca,'FontSize', fs, 'FontName', font_name);

% plot of a single trajectory
figure(); lw = 1.5;
i_min = 20; i_max = 60;
fps = 20;
t_min = i_min / fps; 
t_max = i_max / fps;
t = (i_min:i_max) / fps;
set(gcf, 'Color', [1 1 1]);
subplot(2,1,1);
plot(t, bee_data.raw(17).theta(i_min:i_max), 'LineWidth', lw, 'Color', color.phi);
xlim([t_min, t_max]);
xlabel('Time [s]');
ylabel('\theta [\circ]');
set(gca,'FontSize', fs);
subplot(2,1,2);
plot(t, bee_data.raw(17).Vx(i_min:i_max) / 1000, 'LineWidth', lw, 'Color', color.vy);
xlim([t_min, t_max]);
xlabel('Time [s]');
ylabel('v_x [m/s]');
set(gca,'FontSize', fs, 'FontName', font_name);

% Plot the mean attitude for the bees:
figure();
set(gcf, 'Color', [1 1 1]);
plot(bee_data.references, bee_data.mn_theta, 'LineWidth', lw, 'Color', color.phi);
ylabel('Mean \theta attitude honeybees [\circ]');
xlabel('Velocities [m/s].');
set(gca,'FontSize', fs, 'FontName', font_name);
