function params = set_standard_params_simulate_controller_constant_altitude(MOMENT, RATE_MEASUREMENT, INSECT)
% function params = set_standard_params_simulate_controller_constant_altitude(MOMENT, RATE_MEASUREMENT, INSECT)
%
% Set standard parameters for the simulator. This means no noise, no
% disturbances, etc.
%
% - MOMENT: whether to use a "moment model" with 4 states (rate being one
% of them). If false, a "rate model" will be used with 3 states, regarding
% rate as a control input.
% - RATE_MEASUREMENT: whether the rate is measured (relevant only to the
% moment model).
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('MOMENT', 'var') || isempty(MOMENT))
   MOMENT = false; 
end
if(~exist('RATE_MEASUREMENT', 'var') || isempty(RATE_MEASUREMENT))
   RATE_MEASUREMENT = false; 
end 
if(~exist('INSECT', 'var') || isempty(INSECT))
   INSECT = false; 
end

params.MOMENT = MOMENT;
params.RATE_MEASUREMENT = RATE_MEASUREMENT;

params.DIVERGENCE = false;

if(~MOMENT)
    params.Nx = 3;      % no. of states                         %%%%%%%%%%%
else
    params.Nx = 4;      % no. of states                         %%%%%%%%%%%
end
if(INSECT)
    params.Nx = params.Nx + 1; % add the angle of the wings (head is captured by theta now)
end
params.dt = 1/500;  % sample time simulator and control
params.T = 30;     % max time

% active observability through oscillations:
params.ACTIVE_OSCILLATION = false;
params.period = 1.0; % period of oscillation
params.active_rate_amplitude = 0.2; % amplitude of the active oscillation in rad/s

% use when looping through different sampling times
params.OFdt = 1/30; % OF sample time
params.kd = 0.0; % linear drag term
params.WING_DRAG = false; % whether to vary the drag term significantly with the angle (due to wing surface in the wind)
params.g = 9.81; % gravity vector
params.m = 0.400; % mass
params.g = 9.81; % gravity
params.I = 0.001; % moment of inertia, only relevant for moment model

% Controller gains
params.K_theta = 1.0; % attitude gain
params.K_v = 5*pi/180; % velocity gain
params.K_i = params.K_v; % incremental gain for OF control
params.K_d = 0; % differential gain for OF control
params.K_moment = 10*params.I; % moment gain
params.K_z_factor = 1;
% params.K = 1; %
params.K_head = 0.1; % head attitude gain 

% Parameters Kalman filter:
if(~RATE_MEASUREMENT || ~MOMENT)
    if(~params.DIVERGENCE)
        params.R = diag([0.1].^2);
    else
        params.R = diag([0.1, 0.1].^2);
    end
else
    if(~params.DIVERGENCE)
        params.R = diag([0.1, 0.01].^2);
    else
        params.R = diag([0.1, 0.1, 0.01].^2);
    end
end
if(INSECT)
    n_obs = size(params.R,1);
    params.R(n_obs+1, n_obs+1) = 0.001^2; % very accurate measurement of angle between body and head
end
if(~INSECT)
    if(~MOMENT)
        params.Q = diag([0.2, 2.0 * pi / 180, 0.1].^2); 
        params.P_k1_k1 = diag([1.0, 1.0 * pi / 180, 1.0].^2);  % Initial P - Covariance matrix
    else
        params.Q = diag([0.2, 2.0 * pi / 180, 1.0 * pi / 180, 0.1].^2);
        params.P_k1_k1 = diag([1.0, 1.0 * pi / 180, 1.0 * pi / 180, 1.0].^2);  % Initial P - Covariance matrix    
    end
else
    if(~MOMENT)
        params.Q = diag([0.2, 2.0 * pi / 180, 2.0 * pi / 180, 0.1].^2);
        params.P_k1_k1 = diag([1.0, 1.0 * pi / 180, 1.0 * pi / 180, 1.0].^2);  % Initial P - Covariance matrix
    else
        params.Q = diag([0.2, 2.0 * pi / 180, 1.0 * pi / 180, 2.0 * pi / 180, 0.1].^2);
        params.P_k1_k1 = diag([1.0, 1.0 * pi / 180, 1.0 * pi / 180, 1.0 * pi / 180, 1.0].^2);  % Initial P - Covariance matrix    
    end
end

% noise:
params.sigma_rate = 0.0; %0.02; %deg2rad(1.0); % rate input to system noise
params.sigma_T = 0.0; %0.1; % 0.5; % thrust input to system noise (here just applied as acceleration, so m/s^2)
params.sigma_OF = 0; %0.02; % ventral flow noise
params.sigma_Dv = 0.0; % divergence noise
params.sigma_M = 0.0; % moment noise
params.sigma_rate_msm = 0.0; % rate measurement noise
params.sigma_head_thorax = 0.0; % measurement noise of angle between head and body

% delays, all in seconds:
params.delay_moment = 0;    % delay in moment input to system
params.delay_rate = 0;      % delay in rate input to system
params.delay_rate_k = 0;   % delay in rate input to kalman filter
params.delay_T = 0;      % delay in thrust input to system
params.delay_T_k = 0;   % delay in thrust input to kalman filter
params.delay_OF = 0;    % delay in ventral flow measurements
params.delay_D = 0;     % delay in divergence measurement
params.delay_ht = 0;    % delay in head-thorax measurement

% Controller types:
ATTITUDE_REF = 0;
VELOCITY_REF = 1;
OF_REF = 2;
params.controller_type = OF_REF;

% Control reference over time:
% If size one, a constant reference is followed.
% If size two, at half of the simulation a step will be performed to the
% second reference.
% If longer, it is assumed that a custom reference is given, of the length of the simulation.
params.control_ref = [0.0]; % standard is hover

% External disturbances:
NO_DISTURBANCE = 0;
THRUST_DISTURBANCE = 1;
HORIZONTAL_DISTURBANCE = 2;
MALICIOUS_V0 = 3;
params.external_disturbance = NO_DISTURBANCE;

% whether to plot graphs:
params.graphics = true;
if(~INSECT)
    if(~MOMENT)
        params.initial_state = [0, 0*pi/180, 1]';   %[v, theta, z]
    else
        params.initial_state = [0, 0*pi/180, 0, 1]';   %[v, theta, thetadot, z]
    end
else
    if(~MOMENT)
        params.initial_state = [0, 0*pi/180, 0*pi/180, 1]';   %[v, theta, z]
    else
        params.initial_state = [0, 0*pi/180, 0, 0*pi/180, 1]';   %[v, theta, thetadot, z]
    end    
end
params.initial_estimated_state = params.initial_state;

% whether to include actuator dynamics in case of rate commands:
params.ACTUATOR_DYNAMICS = false;
params.alpha = 0.7; % 0.0 = no actuator delay, 1.0 = command has no effect

% Simulations with a slope:
params.LANDSCAPE = false; % default is flat ground
params.landscape_params = [0.5]; % height = a * pos, a single param indicates the slope a 
params.estimate_slope = false;

% do not use the Kalman filter's estimate, but the ground truth with noise
params.USE_NOISY_GROUND_TRUTH = false;
params.std_GT_attitude_noise = deg2rad(1);
params.SCALE_RATE_INSTABILITY = false;

% Whether to make a video:
params.VIDEO = false;
params.video_dt = 0.1;
params.save_video = false;

% How to integrate the differential equations. If false, Euler integration
% is used.
params.ODE45 = false;