function [metrics, results] = simulate_controller_constant_altitude_changing_attitude_insect(params)
% function [metrics, results] = simulate_controller_constant_altitude_changing_attitude_insect(params)
% 
% Simulate controller based on the estimator - velocity in x (vx), constant altitude z, changing attitude phi, p only
% 
%     X = [vx phi phi_wings z]';             % State Vector
%     f = [(g*tan(phi_wings)); p; p_wings; 0];   % State Derivative
%     zm = [-vx*(cos(phi))^2/z + p; phi - phi_wings];   % State measurement
%
% - params: structure containing all parameters for the experiment. If not
% provided, the standard parameters will be used.
% - metrics: control performance metrics.
% - results: states over time, etc.
%
% Article:
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('params', 'var') || isempty(params))
    % Get insect parameters for a rate model:
    params = set_standard_params_simulate_controller_constant_altitude(false, false, true);
end

% x=[xd; theta; z]
xd_ind = 1;
theta_ind = 2;
theta_wings_ind = 3;
z_ind = 4;
alpha_ind = 5; % for estimating slope

%% Copy parameter settings

% whether to show a video of the simulation
VIDEO = params.VIDEO;

% whether the ground surface has a slope:
LANDSCAPE = params.LANDSCAPE;
landscape_params = params.landscape_params;
estimate_slope = params.estimate_slope;

initial_state = params.initial_state;
n_states = params.Nx;      % no. of states                         %%%%%%%%%%%
n_states_filter = n_states;
if(LANDSCAPE)
    n_states = n_states + 1; % also keep track of position in the sim, not in the filter
    initial_state = [initial_state; 0]; % set pos to zero without losing generality
    if(estimate_slope)
        % estimate the slope:
        n_states_filter = n_states;
    end
end

dt = params.dt;  % sample time simulator and control
T = params.T;     % max time

% active observability through oscillations:
ACTIVE_OSCILLATION = params.ACTIVE_OSCILLATION;
period = params.period; % period of oscillation
active_rate_amplitude = params.active_rate_amplitude; % amplitude of the active oscillation in rad/s

% use when looping through different sampling times
OFdt = params.OFdt; % OF sample time
kd = params.kd; % linear drag term 
kd0  = kd;
g = params.g; % gravity vector
m = params.m; % mass
g = params.g; % gravity

% Controller gains
K_theta = params.K_theta; % attitude gain
K_v = params.K_v; % velocity gain
K_i = params.K_i; % velocity gain
K_d = params.K_d; % velocity gain
K_head = params.K_head; % gain for head control

% noise:
sigma_rate = params.sigma_rate; % rate input to system noise
sigma_T = params.sigma_T; % thrust input to system noise (here just applied as acceleration, so m/s^2)
sigma_OF = params.sigma_OF; % ventral flow noise
sigma_Dv = params.sigma_Dv; % divergence noise
sigma_ht = params.sigma_head_thorax; % measuring angle between head and body noise
sigma_GT_att = params.std_GT_attitude_noise; % when using noisy ground truth for control

% delays, all in time steps:
delay_rate = round(params.delay_rate / dt);      % delay in rate input to system
delay_rate_k = round(params.delay_rate_k / dt);   % delay in rate input to kalman filter
delay_T = round(params.delay_T / dt);      % delay in thrust input to system
delay_T_k = round(params.delay_T_k / dt);   % delay in thrust input to kalman filter
delay_OF = round(params.delay_OF / dt);    % delay in ventral flow measurements
delay_D = round(params.delay_D / dt);     % delay in divergence measurement
delay_ht = round(params.delay_ht / dt);   % delay in head-thorax measurement
 
% EKF parameters: 
R = params.R;
Q = params.Q;
P_k1_k1 = params.P_k1_k1;  % Initial P - Covariance matrix
if(estimate_slope)
    P_k1_k1(alpha_ind, alpha_ind) = (20/180)*pi;
    Q(alpha_ind, alpha_ind) = (2/180)*pi;
end

% Controller type:
ATTITUDE_REF = 0;
VELOCITY_REF = 1;
OF_REF = 2;
controller_type = params.controller_type;

% External disturbances:
NO_DISTURBANCE = 0;
THRUST_DISTURBANCE = 1;
HORIZONTAL_DISTURBANCE = 2;
MALICIOUS_V0 = 3;
external_disturbance = params.external_disturbance;

% Whether it takes time to reach a desired rate:
ACTUATOR_DYNAMICS = params.ACTUATOR_DYNAMICS;
alpha = params.alpha;

% Whether to include optic flow divergence:
DIVERGENCE = params.DIVERGENCE;

% Whether to vary the drag with angle:
WING_DRAG = params.WING_DRAG;

%%  Initialize variables:

t = 0:dt:T; % simulation time steps
N = length(t);
Nidx =  ceil(OFdt/dt);
OFidx = 1:Nidx:N;

% Control reference:
if(length(params.control_ref) == 1)
    % constant reference:
    control_ref = params.control_ref(1) * ones(1, N);
elseif(length(params.control_ref) == 2)
    % step in the reference halfway the simulation:
    control_ref = [params.control_ref(1) * ones(1, ceil(N/2)), params.control_ref(2) * ones(1, floor(N/2))];
else
    control_ref = params.control_ref;
    if(length(control_ref) ~= N)
        error('control_ref is not of the proper length!');
    end
end

% states:
x = zeros(n_states, N);
x(:,1) = initial_state;   %[v, theta, theat_wings, z]
X_k1_k1 = params.initial_estimated_state; 
if(estimate_slope)
    X_k1_k1 = [X_k1_k1; 0]; 
end
Cov = zeros(n_states_filter, N);

% noise over time (predetermined for the entire run):
ratev = sigma_rate*randn(1, N);
Tv = sigma_T*randn(1, N);
disturbance = ones(1, N); 
OFv = sigma_OF*randn(1, N);
Dv = sigma_Dv*randn(1, N);
htv = sigma_ht*randn(1, N);
att_GT_v = sigma_GT_att*randn(1, N);

% Kalman Filter Initialization
Xe = zeros(n_states_filter, N);   % Estimated states
std_Xe = zeros(n_states_filter, N);   % Standard deviation in estimated state values
G = eye(n_states_filter);
COi = zeros(1, N);     % Degree of observability
T = zeros(1, N);       % Thrust input to system
T_k = zeros(1, N);       % Thrust input to EKF
rate = zeros(1, N);       % Rate input to system (thorax)
rate_k = zeros(1, N);     % Rate input to EKF
rate_head = zeros(1, N);    % Rate input to the head 
OF_e = zeros(1, N);    % undelayed OF output 
GT_OF_e = zeros(1, N);    % undelayed ground truth OF output (without noise)
Kalman_gain = zeros(n_states_filter, N);
Cov(:, 1) = diag(P_k1_k1);
rate_c = 0;
rate_c_head = 0;
OF_err = 0;
lp_OF_err = 0;
incr_OF_err = 0;

if(~DIVERGENCE)
    z = zeros(2, N); % observed flow
else
    z = zeros(3, N); % observed flow and divergence
end

%% Simulate, with k as time step variable:

if(VIDEO)
    h_video = figure();
    video_frame = 0;
    
    if(params.save_video && ~exist('./video', 'dir'))
        mkdir('./video');
    end
end

for k = 1:1:N-1
   
    % show a video:
    if(VIDEO && mod(k*dt, params.video_dt) == 0)
        clf('reset')
        draw_state_const_alt_changing_att_insect(x(:,k), LANDSCAPE, landscape_params, k*dt); 
        drawnow;
        pause(0.1);
        if(params.save_video)
            % save the frame
            saveas(h_video, ['./video/image_' num2str(video_frame) '.png']);
            video_frame = video_frame + 1;
        end
    end
    
    if(find(OFidx == k+1))    % Check if OF is sampled at current instant
        % if OF sampled perform measurement correction during estimation
        flag = 1; 
        % Determine optical flow measurement: 
        if(~LANDSCAPE)
            OF_out = OF_output_const_alt_changing_att_insect(x(:, k), rate_c_head, OFv(k), Dv(k));
        else
            OF_out = OF_output_const_alt_changing_att_landscape_insect(x(:, k), rate_c_head, OFv(k), Dv(k), landscape_params);
        end
        % Gives measurements
        if(k+1+delay_OF < N)
            z(1, k+1+delay_OF) = OF_out(1);
            if(DIVERGENCE)
                z(2, k+1+delay_OF) = OF_out(2); 
                z(3, k+1+delay_ht) = x(2, k) - x(3, k) + htv(k); 
            else
                z(2, k+1+delay_ht) = x(2, k) - x(3, k) + htv(k);
            end
        end
    else
        flag = 0;
        if(k+1+delay_OF < N)
            z(1, k+1+delay_OF) = z(1, k+delay_OF); % equal to previous time step
            if(DIVERGENCE)
                z(2, k+1+delay_OF) = z(2, k+delay_OF); 
                z(3, k+1+delay_ht) = z(3, k+delay_ht); 
            else
                z(2, k+1+delay_ht) = z(2, k+delay_ht);
            end
        end
    end
    
    % Controller
    if(controller_type == VELOCITY_REF)
        % velocity control:
        theta_ref = K_v * (control_ref(k) - Xe(1, k)); % P-control of velocity control
        theta_ref = min([deg2rad(45), theta_ref]);
        theta_ref = max([-deg2rad(45), theta_ref]);
        thetad_ref = K_theta*(theta_ref - Xe(3, k)); % P-control of thorax attitude control
        
        thetad_head_ref = K_head * (0 - Xe(2, k)); % P-control of the head to stay level
        
    elseif(controller_type == OF_REF)
        % optical flow control:     
        OF_err = control_ref(k) - z(1, k);
        lp_OF_err_new = 0.95 * lp_OF_err + 0.05 * OF_err;
        dt_OF_err = lp_OF_err_new - lp_OF_err;
        lp_OF_err = lp_OF_err_new;
        incr_OF_err = incr_OF_err + OF_err;
        theta_ref = -K_v * (OF_err) -K_i * dt * incr_OF_err + (1/dt) * K_d * dt_OF_err; % P,I,D-control of ventral optical flow
        
        theta_ref = min([deg2rad(70), theta_ref]);
        theta_ref = max([-deg2rad(70), theta_ref]);
        
        if(params.SCALE_RATE_INSTABILITY)
            v_factor = 0.1 + 0.01 * abs(x(1, k));
            v_factor = min([v_factor, 10]);
        else
            v_factor = 1;
        end
        if(~params.USE_NOISY_GROUND_TRUTH)
            % standard case, use the estimate:
            thetad_ref = v_factor * K_theta*(theta_ref - Xe(3, k)); % P-control of thorax attitude control
            thetad_head_ref = K_head * (0 - Xe(2, k)); % P-control of the head to stay level
        else
            % use a noisy ground truth:
            thetad_ref = v_factor * K_theta*(theta_ref - x(3, k) - att_GT_v(k)); % P-control of thorax attitude control
            thetad_head_ref = K_head * (0 - x(2, k) - att_GT_v(k)); % P-control of the head to stay level
        end
        
        
    elseif(controller_type == ATTITUDE_REF)
        % attitude control:
        thetad_ref = K_theta*(control_ref(k) - Xe(3, k)); % P-control of thorax attitude control
        
        thetad_head_ref = K_head * (0 - Xe(2, k)); % P-control of the head to stay level
    end
    
    % whether to actively induce observability by oscillating:
    if(ACTIVE_OSCILLATION)
        curr_t = t(k);
        add_rate = active_rate_amplitude * sin(curr_t*(2*pi)/period);
        rate_c = thetad_ref + add_rate;
    else
        rate_c = thetad_ref;
    end
    
    rate_c_head = thetad_head_ref; % no active oscillations here yet ; )
    
    if(~ACTUATOR_DYNAMICS)
        rate(k + delay_rate) = rate_c + ratev(1, k); % Commanded rate that will be delayed due to actuator delay
        rate_k(k + delay_rate_k) = rate_c; % Commanded rate as input to filter that will be delayed due to actuator delay
     
        rate_head(k + delay_ht) = rate_c_head; % no actuation noise on the head for now
    else
        rate(k + delay_rate) = (1-alpha) * rate_c + alpha * rate(k + delay_rate - 1) + ratev(1, k); % Commanded rate that will be delayed due to actuator delay
        rate_k(k + delay_rate_k) = (1-alpha) * rate_c + alpha * rate(k + delay_rate - 1); % Commanded rate as input to filter that will be delayed due to actuator delay
        
        rate_head(k + delay_ht) = rate_c_head; % no actuation noise & actuator dynamics on the head for now
    end

    %% External disturbances:
    if(external_disturbance == NO_DISTURBANCE)
        noise_ax = 0;
        noise_az = 0;
    elseif(external_disturbance == THRUST_DISTURBANCE)
        % actual thrust noise:
        theta = x(2, k);
        noise_ax = Tv(k) * sin(theta);
        noise_az = Tv(k) * cos(theta);
    elseif(external_disturbance == HORIZONTAL_DISTURBANCE)
        % disturbance in y axis:
        if(disturbance(k))
            noise_ax = Tv(k);
        else
            noise_ax = 0;
        end
        noise_az = 0;
    elseif(external_disturbance == MALICIOUS_V0)
        % disturbance purely in z axis: malicious for the v=0 condition
        noise_ax = 0;
        if(disturbance(k) && abs(x(1,k)) < 1e-2) % 1e-3
            noise_az = 1.0; % only upward disturbances
        else
            noise_az = 0;
        end
    end
    
    if(WING_DRAG)
        % change the drag factor with the angle:
        kd = get_wing_drag_factor(x(3,k), kd0);
    end
    
    %% Simulate Quadcopter Dynamics
    if(params.ODE45)
        % ODE45:
        ode_options = odeset('RelTol', 1e-8, 'AbsTol', 1e-8);
        tspan = [t(k), t(k+1)];
        tt = t(k); xx = x(:,k);
        if(~LANDSCAPE)
            [tt,ode_out] = ode45(@(tt,xx) odefcn_const_alt_changing_att_insect(tt, xx, m, g, rate(k), rate_head(k), kd, [noise_ax 0 0 noise_az]), tspan, xx, ode_options); 
        else
            [tt,ode_out] = ode45(@(tt,xx) odefcn_const_alt_changing_att_landscape(tt, xx, m, g, rate(k), kd, [noise_ax 0 0 noise_az]), tspan, xx, ode_options);
        end
        x(:, k + 1) = ode_out(end, :)';
    else
        % EULER:
        if(~LANDSCAPE)
            x(:, k + 1) =  x(:, k) + odefcn_const_alt_changing_att_insect(1, x(:, k), m, g, rate(k), rate_head(k), kd, [noise_ax 0 0 noise_az])*(t(k+1)-t(k)); % array with zeros is actuation noise
        else
            x(:, k + 1) =  x(:, k) + odefcn_const_alt_changing_att_landscape(1, x(:, k), m, g, rate(k), kd, [noise_ax 0 0 noise_az])*(t(k+1)-t(k)); % array with zeros is actuation noise
        end
    end
    
    %% Run EKF to estimate states
    if(find(OFidx == k+1))    % Check if OF is sampled at current instant
        % if OF sampled perform measurement correction during estimation
        flag = 1;
        if(~LANDSCAPE || ~estimate_slope)
            [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_insect(X_k1_k1, P_k1_k1, z(:, k+1), m, g, rate_k(k), rate_head(k), kd, t(k), t(k+1), G, Q, R, flag);
        else
            [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_landscape(X_k1_k1, P_k1_k1, z(:, k+1), m, g, rate_k(k), kd, t(k), t(k+1), G, Q, R, flag);
        end
    else
        % if OF is not sampled skip measurement correction
        flag = 0;
        if(size(z, 1) == 2)
            zero_measurement = [0; 0];
        else
            zero_measurement = [0; 0; 0];
        end
        if(~LANDSCAPE || ~estimate_slope)
            [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_insect(X_k1_k1, P_k1_k1, zero_measurement, m, g, rate_k(k), rate_head(k), kd, t(k), t(k+1), G, Q, R, flag);
        else
            [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_landscape(X_k1_k1, P_k1_k1, zero_measurement, m, g, rate_k(k), kd, t(k), t(k+1), G, Q, R, flag);
        end
    end
    Kalman_gain(:, k+1) = K_k1(:, 1);
    
    %storage
    Xe(:, k+1) = X_k1_k1;
    if(flag == 1)
        OF_e(:, k+1) = OF_out(1);
    else
        OF_e(:, k+1) = OF_e(:, k);
    end
    GT_OF_out = OF_output_const_alt_changing_att_insect(x(:, k), rate_c_head, 0, 0);
    GT_OF_e(:, k+1) = GT_OF_out(1);
    Cov(:, k+1) = diag(P_k1_k1);
    % Observability calculator
    COi(1, k) = observability_deg_constant_alt_changing_att_insect(x(:,k), rate_c_head, rate_c + ratev(1, k), g, kd, m, DIVERGENCE);
    
end

%% Error metrics:
if(controller_type == VELOCITY_REF)
    GT_control_err = x(1,1:end)-control_ref(1,1:end);
    observed_control_err = Xe(1,1:end)-control_ref(1,1:end);
    observed = Xe(1,1:end);
    GT = x(1,1:end);
elseif(controller_type == ATTITUDE_REF)
    GT_control_err = x(3,1:end)-control_ref(1,1:end);
    observed_control_err = Xe(3,1:end)-control_ref(1,1:end);
    observed = Xe(3,1:end);
    GT = x(3,1:end);
else
    GT_control_err = GT_OF_e - control_ref(1,1:end);
    observed_control_err = z(1,:) - control_ref(1,1:end);
    observed = z(1,:);
    GT = GT_OF_e;
end

% MAE during whole run:
metrics.MAE_GT = mean(abs(GT_control_err));
metrics.MAE = mean(abs(observed_control_err));

% During last N seconds
N_sec = 1;
metrics.N_sec = N_sec;
n_steps_sec = round(1/dt);
metrics.MAE_GT_last_N_sec = mean(abs(GT_control_err(end-N_sec*n_steps_sec:end)));
metrics.MAE_last_N_sec = mean(abs(observed_control_err(end-N_sec*n_steps_sec:end)));

% In case of step references:
if(length(params.control_ref) == 2)
    if(params.control_ref(2) > params.control_ref(1))
        POS_PEAKS = true;
    else
        POS_PEAKS = false;
    end
    [rise_time, percentage_overshoot, mean_absolute_error] = get_performance_characteristics( ...
        smooth(GT(round(N/2):end), n_steps_sec), control_ref(end), t(round(N/2):end)-t(round(N/2)), POS_PEAKS); 
    metrics.rise_time = rise_time;
    metrics.percentage_overshoot = percentage_overshoot;
    metrics.mean_absolute_error = mean_absolute_error;
end

results.state = x;
results.estimated_state = Xe;
results.observability = COi;
results.time = t;
results.rate = rate_k;

%% Plotting:
if(params.graphics)
    
    color = get_color_palette();
    font_info = get_font_info();
    
    dark_factor_gt = 0.5;
    
    lw = 1.2;
    lstyle_gt = '-.';
    lw_gt = 1;
    lw_ref = 1.2;
    
    figure('name', 'Overview plot');
    set(gcf, 'Color', [1 1 1]);
    % VELOCITY:
    subplot(2,3,1);
    plot(t(1:end), Xe(1,1:end), 'LineWidth',lw, 'Color', color.vy);
    hold on;
    plot(t(1:end), x(1,1:end), lstyle_gt, 'LineWidth',lw_gt, 'Color', dark_factor_gt * color.vy);
    if(controller_type == VELOCITY_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth',lw_ref);
    end
    plot([t(1), t(end)], [0 0], 'k:');
    hold off;
    xlabel('Time (s)');
    ylabel('$v$ (m/s)', 'interpreter', 'latex');
    if(controller_type == VELOCITY_REF)
        legend('Estimated', 'True', 'Ref')
    else
        legend('Estimated', 'True')
    end
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    % ATTITUDE:
    subplot(2,3,2);
    plot(t(1:end), Xe(3,1:end)*180/pi, 'LineWidth',lw, 'Color', color.phi)
    hold on;
    plot(t(1:end), x(3,1:end)*180/pi, lstyle_gt, 'LineWidth', lw_gt, 'Color', dark_factor_gt * color.phi)
    % also plot head:
    plot(t(1:end), Xe(2,1:end)*180/pi, 'LineWidth',lw, 'Color', color.phi_head)
    plot(t(1:end), x(2,1:end)*180/pi, lstyle_gt, 'LineWidth', lw_gt, 'Color', dark_factor_gt * color.phi_head)
    if(controller_type == ATTITUDE_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth', lw_ref);
    end
    plot([t(1), t(end)], [0 0], 'k:');
    hold off;
    xlabel('Time (s)');
    ylabel('\phi (deg)');
    if(controller_type == ATTITUDE_REF)
        legend('Estimated body', 'True body', 'Estimated head', 'True head', 'Ref')
    else
        legend('Estimated body', 'True body', 'Estimated head', 'True head')
    end
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    % Height:
    subplot(2,3,3);
    plot(t(1:end), Xe(4,1:end), 'LineWidth', lw, 'Color', color.z)
    hold on;
    if(~LANDSCAPE)
        plot(t(1:end), x(4,1:end), lstyle_gt, 'LineWidth', lw_gt, 'Color', dark_factor_gt * color.z)
    else
        plot(t(1:end), x(4,1:end)-x(5,1:end).*landscape_params(1), lstyle_gt, 'LineWidth', lw_gt, 'Color', dark_factor_gt * color.z)
    end
    % plot(t(1:end), h_ref(1,1:end),'--r','LineWidth',1.2);
    hold off;
    xlabel('Time (s)');
    ylabel('$z$ (m)', 'interpreter', 'latex');
    legend('Estimated', 'True');
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    % VENTRAL FLOW:
    subplot(2,3,4);
    plot(t(1:end), z(1, 1:N), 'LineWidth', lw, 'Color', color.flow);
    hold on;
    plot(t(1:end), OF_e(1, 1:end), ':', 'LineWidth', lw, 'Color', color.flow);
    plot(t(1:end), GT_OF_e(1, 1:end), lstyle_gt, 'LineWidth', lw_gt, 'Color', dark_factor_gt * color.flow);
    if(controller_type == OF_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth', lw_ref);
    end
    xlim([t(1) t(end)])
    xlabel('Time (s)');
    ylabel('Optical flow (rad/s)');
    if(controller_type == OF_REF)
        legend('Measured (delayed)', 'Measured (undelayed)', 'Ground truth (undelayed)', 'Ref');
    else
        legend('Measured (delayed)', 'Measured (undelayed)', 'Ground truth (undelayed)');
    end
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    % Rate:
    subplot(2,3,5);
    plot(t(1:end), rate(1, 1:N), 'LineWidth', lw, 'Color', color.gyro);
    hold on;
    plot(t(1:end), rate_head(1, 1:N), 'LineWidth', lw, 'Color', color.gyro_head);
    plot([t(1), t(end)], [0 0], 'k:');
    xlim([t(1) t(end)])
    xlabel('Time (s)');
    ylabel('Rate (rad/s)');
    legend('Rate body', 'Rate head');
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    % Observability
    subplot(2,3,6);
    plot(t(1:end), COi(1, 1:N), 'LineWidth', lw, 'Color', color.observability_sim);
    xlim([t(1) t(end)])
    xlabel('Time (s)');
    ylabel('Observability degree (-)');
    legend('Observability degree');
    set(gca,'FontSize', font_info.size, 'FontName', font_info.name);
    
    %Kalman gain
    figure('name', 'Kalman gain over time');
    set(gcf, 'Color', [1 1 1]);
    plot(t(1:end), Kalman_gain(1,1:end), 'LineWidth',1.2);
    hold on;
    plot(t(1:end), Kalman_gain(2,1:end), 'LineWidth',1.2);
    plot(t(1:end), Kalman_gain(3,1:end), 'LineWidth',1.2);
    plot(t(1:end), Kalman_gain(4,1:end), 'LineWidth',1.2);
    xlabel('time (s)');
    ylabel('v (m/s)');
    legend('K v', 'K \phi head', 'K \phi wings', 'K Z')
    set(gca,'FontSize', 16);
    
    if(LANDSCAPE && estimate_slope)
        figure('name', 'Slope')
        set(gcf, 'Color', [1 1 1]);
        plot(t, rad2deg(Xe(alpha_ind, :)));
        hold on;
        plot([t(1), t(end)], [rad2deg(atan(landscape_params(1))), rad2deg(atan(landscape_params(1)))], 'g--');
        xlabel('time (s)');
        ylabel('slope (\circ)');
        legend('\hat{\alpha}', '\alpha^*');
        set(gca,'FontSize', 16);
    end
    
    % ESTIMATION ERRORS OVER TIME
    figure('name', 'Absolute estimation errors over time');
    set(gcf, 'Color', [1 1 1]);
    % VELOCITY:
    subplot(1,3,1);
    plot(t(1:end), abs(Xe(1,1:end)-x(1,1:end)), 'LineWidth',1.2);
    xlabel('time (s)');
    ylabel('$|\hat{v} - v|$ (m/s)','interpreter','latex');
    set(gca,'FontSize', 16);
    
    % ATTITUDE:
    subplot(1,3,2);
    plot(t(1:end), abs(Xe(2,1:end)*180/pi - x(2,1:end)*180/pi), 'LineWidth',1.2)
    hold on
    plot(t(1:end), abs(Xe(3,1:end)*180/pi - x(3,1:end)*180/pi), 'LineWidth',1.2)
    xlabel('time (s)');
    ylabel('$|\hat{\phi} - \phi|$ (deg)','interpreter','latex');
    legend('Head', 'Body');
    set(gca,'FontSize', 16);
    
    % Height:
    subplot(1,3,3);
    if(~LANDSCAPE)
        plot(t(1:end), abs(Xe(4,1:end) - x(4,1:end)), 'LineWidth',1.2)
    else
        plot(t(1:end), abs(Xe(4,1:end) - (x(4,1:end)-x(5,1:end).*landscape_params(1))), 'LineWidth',1.2)
    end
    xlabel('time (s)');
    ylabel('$|\hat{h} - h|$ (m)','interpreter','latex');
    set(gca,'FontSize', 16);
end