function [metrics, results] = simulate_controller_constant_altitude_changing_attitude_moment(params)
% function [metrics, results] = simulate_controller_constant_altitude_changing_attitude_moment(params)
% 
% Simulate controller based on the estimator - velocity in x (vx), constant altitude z, changing attitude phi, p only
% 
%     X = [vx phi phidot z]';             % State Vector
%     f = [(g*tan(phi)); p; M/I; 0];   % State Derivative
%     zm = [-vx*(cos(phi))^2/z + p, p];   % State measurement
%
% - params: structure containing all parameters for the experiment. If not
% provided, the standard parameters will be used.
% - metrics: control performance metrics.
% - results: contains all states over time, etc.
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('params', 'var') || isempty(params))
    params = set_standard_params_simulate_controller_constant_altitude(true);
end

%% Copy parameter settings
% whether to make a video of the simulation:
VIDEO = params.VIDEO;

% whether to measure the rates:
RATE_MEASUREMENT = params.RATE_MEASUREMENT;

Nx = params.Nx;      % no. of states                         %%%%%%%%%%%
dt = params.dt;  % sample time simulator and control
T = params.T;     % max time

% active observability through oscillations:
ACTIVE_OSCILLATION = params.ACTIVE_OSCILLATION;
period = params.period; % period of oscillation
active_rate_amplitude = params.active_rate_amplitude; % amplitude of the active oscillation in rad/s

% use when looping through different sampling times
OFdt = params.OFdt; % OF sample time
if(OFdt < dt)
    % if the optical flow update time is smaller than the simulation time,
    % make the simulation more fine-grained:
    dt = OFdt;
end
kd = params.kd; % linear drag term
g = params.g; % gravity vector
m = params.m; % mass
g = params.g; % gravity
I = params.I; % moment of inertia

% Controller gains
K_theta = params.K_theta; % attitude gain
K_v = params.K_v; % velocity gain
K_moment = params.K_moment; % moment gain

% noise:
sigma_rate = params.sigma_rate; % rate input to system noise
sigma_T = params.sigma_T; % thrust input to system noise (here just applied as acceleration, so m/s^2)
sigma_OF = params.sigma_OF; % ventral flow noise
sigma_Dv = params.sigma_Dv; % divergence noise
sigma_M = params.sigma_M; % moment noise
sigma_rate_msm = params.sigma_rate_msm; % rate measurement noise

% delays, all in time steps:
delay_rate = round(params.delay_rate / dt);      % delay in rate input to system
delay_rate_k = round(params.delay_rate_k / dt);   % delay in rate input to kalman filter
delay_T = round(params.delay_T / dt);      % delay in thrust input to system
delay_T_k = round(params.delay_T_k / dt);   % delay in thrust input to kalman filter
delay_OF = round(params.delay_OF / dt);    % delay in ventral flow measurements
delay_D = round(params.delay_D / dt);     % delay in divergence measurement
delay_moment = round(params.delay_moment / dt);     % delay in moment input
delay_moment_k = delay_moment;

% EKF parameters: will not assume zero noise and delay in the filtering
R = params.R;
Q = params.Q;
P_k1_k1 = params.P_k1_k1;  % Initial P - Covariance matrix

% Controller type:
ATTITUDE_REF = 0;
VELOCITY_REF = 1;
OF_REF = 2;
controller_type = params.controller_type;

% External disturbances:
NO_DISTURBANCE = 0;
THRUST_DISTURBANCE = 1;
HORIZONTAL_DISTURBANCE = 2;
MALICIOUS_V0 = 3;
external_disturbance = params.external_disturbance;

%%  Initialize variables:

t = 0:dt:T; % simulation time steps
N = length(t);
Nidx =  ceil(OFdt/dt);
OFidx = 1:Nidx:N;

% x=[xd; theta; thetadot; z]
xd_ind = 1;
theta_ind = 2;
thetad_ind = 3;
z_ind = 4;

% Control reference:
if(length(params.control_ref) == 1)
    % constant reference:
    control_ref = params.control_ref(1) * ones(1, N);
elseif(length(params.control_ref) == 2)
    % step in the reference halfway the simulation:
    control_ref = [params.control_ref(1) * ones(1, ceil(N/2)), params.control_ref(2) * ones(1, floor(N/2))];
else
    control_ref = params.control_ref;
    if(length(control_ref) ~= N)
        error('control_ref is not of the proper length!');
    end
end

% states:
x = zeros(Nx, N);
if(~RATE_MEASUREMENT)
    z = zeros(1, N); % 1 observation
else
    z = zeros(2, N); % 2 observations
end
x(:,1) = params.initial_state;   %[v, theta, thetad, z]
X_k1_k1 = params.initial_estimated_state; 
Cov = zeros(Nx, N);

% noise over time:
ratev = sigma_rate*randn(1, N);
Tv = sigma_T*randn(1, N);
disturbance = ones(1, N); 
OFv = sigma_OF*randn(1, N);
Dv = sigma_Dv*randn(1, N);
momentv = sigma_M*randn(1,N);
ratemsmv = sigma_rate_msm*randn(1,N);

% Kalman Filter Initialization
Xe = zeros(Nx, N);   % Estimated states
std_Xe = zeros(Nx, N);   % Standard deviation in estimated state values
G = eye(Nx);
COi = zeros(1, N);     % Degree of observability
T = zeros(1, N);       % Thrust input to system
T_k = zeros(1, N);       % Thrust input to EKF
moment = zeros(1, N);       % Moment input to system
moment_k = zeros(1, N);     % Moment input to EKF
OF_e = zeros(1, N);    % undelayed OF output 
GT_OF_e = zeros(1, N);    % undelayed ground truth OF output (without noise)
Kalman_gain = zeros(Nx, N);
Cov(:, 1) = diag(P_k1_k1);
rate_c = 0;
if(RATE_MEASUREMENT)
    z = zeros(2, N); % observed flow, rate measurement
else
    z = zeros(1, N); % observed flow
end

%% Simulate, with k as time step variable:

if(VIDEO)
    h_video = figure();
    video_frame = 0;
    if(params.save_video && ~exist('./video', 'dir'))
        mkdir('./video');
    end
end

for k = 1:1:N-1
    
    if(VIDEO && mod(k*dt, params.video_dt) == 0)
        % draw the new state in the figure:
        clf('reset')
        draw_state_const_alt_changing_att(x(:,k), LANDSCAPE, landscape_params, k*dt);
        drawnow;
        pause(0.1);
        if(params.save_video)
            % save the figure to file:
            saveas(h_video, ['./video/image_' num2str(video_frame) '.png']);
            video_frame = video_frame + 1;
        end
    end
    
    if(find(OFidx == k+1))    % Check if OF is sampled at current instant
        % if OF sampled perform measurement correction during estimation
        flag = 1;
        % Determine optical flow measurement: 
        OF_out = OF_output_const_alt_changing_att_moment(x(:, k), OFv(k), Dv(k));
        % Gives measurements
        if(k+1+delay_OF < N)
            z(1, k+1+delay_OF) = OF_out(1);
            %z(2, k+1+delay_D) = OF_out(2); 
        end
    else
        flag = 0;
        if(k+1+delay_OF < N)
            z(1, k+1+delay_OF) = z(1, k+delay_OF); % equal to previous time step
            %z(2, k+1+delay_D) = OF_out(2); 
        end
    end
    if(RATE_MEASUREMENT)
        % Gaussian noise on gyro, no bias
        z(2, k+1) = x(thetad_ind, k) + ratemsmv(k);
    end
    
    % Controller
    if(controller_type == VELOCITY_REF)
        % velocity control:
        theta_ref = K_v * (control_ref(k) - Xe(1, k)); % P-control of velocity control
        theta_ref = min([deg2rad(45), theta_ref]);
        theta_ref = max([-deg2rad(45), theta_ref]);
        thetad_ref = K_theta*(theta_ref - Xe(2, k)); % P-control of attitude control
    elseif(controller_type == OF_REF)
        % optical flow control:
        theta_ref = -K_v * (control_ref(k) - z(1, k)); % P-control of ventral optical flow
        theta_ref = min([deg2rad(45), theta_ref]);
        theta_ref = max([-deg2rad(45), theta_ref]);
        thetad_ref = K_theta*(theta_ref - Xe(2, k)); % P-control of attitude control
    elseif(controller_type == ATTITUDE_REF)
        % attitude control:
        thetad_ref = K_theta*(control_ref(k) - Xe(2, k)); % P-control of attitude control
    end
    
    % whether to actively induce observability by oscillating:
    if(ACTIVE_OSCILLATION)
        curr_t = t(k);
        add_rate = active_rate_amplitude * sin(curr_t*(2*pi)/period);
        rate_c = thetad_ref + add_rate;
    else
        rate_c = thetad_ref;
    end
    
    moment_c = K_moment*(rate_c - Xe(3, k));
    moment(k + delay_moment) = moment_c + momentv(1, k); % Commanded rate that will be delayed due to actuator delay
    moment_k(k + delay_moment_k) = moment_c; % Commanded rate as input to filter that will be delayed due to actuator delay
    
    %% External disturbances:
    if(external_disturbance == NO_DISTURBANCE)
        noise_ax = 0;
        noise_az = 0;
    elseif(external_disturbance == THRUST_DISTURBANCE)
        % actual thrust noise:
        theta = x(2, k);
        noise_ax = Tv(k) * sin(theta);
        noise_az = Tv(k) * cos(theta);
    elseif(external_disturbance == HORIZONTAL_DISTURBANCE)
        % disturbance in x axis:
        if(disturbance(k))
            noise_ax = Tv(k);
        else
            noise_ax = 0;
        end
        noise_az = 0;
    elseif(external_disturbance == MALICIOUS_V0)
        % disturbance purely in z axis: malicious for the v=0 condition
        noise_ax = 0;
        if(disturbance(k) && abs(x(1,k)) < 1e-3)
            noise_az = 1.0; % only upward disturbances
        else
            noise_az = 0;
        end
    end
    
    %% Simulate Quadcopter Dynamics
    
    if(params.ODE45)
        % ODE45:
        ode_options = odeset('RelTol', 1e-8, 'AbsTol', 1e-8);
        tspan = [t(k), t(k+1)];
        tt = t(k); xx = x(:,k);
        [tt,ode_out] = ode45(@(tt,xx) odefcn_const_alt_changing_att_moment(tt, xx, m, g, moment(k), I, kd, [noise_ax 0 0 noise_az]), tspan, xx, ode_options);
        x(:, k + 1) = ode_out(end, :)';
    else
        % EULER:
        x(:, k + 1) =  x(:, k) + odefcn_const_alt_changing_att_moment(1, x(:, k), m, g, moment(k), I, kd, [noise_ax 0 0 noise_az])*(t(k+1)-t(k));
    end
    
    %% Run EKF to estimate states
    if(find(OFidx == k+1))    % Check if OF is sampled at current instant
        % if OF sampled perform measurement correction during estimation
        flag = 1;
        [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_moment(X_k1_k1, P_k1_k1, z(:, k+1), m, g, moment_k(k), I, kd, t(k), t(k+1), G, Q, R, flag);
    else
        flag = 0;
        [X_k1_k1, P_k1_k1, i_k1_k1, K_k1] = kalman_filter_constant_alt_changing_att_moment(X_k1_k1, P_k1_k1, z(:, k+1), m, g, moment_k(k), I, kd, t(k), t(k+1), G, Q, R, flag);
    end
    Kalman_gain(:, k+1) = K_k1(:,1); % keep track of gain for other observables?
    
    %storage
    Xe(:, k+1) = X_k1_k1;
    if(flag == 1)
        OF_e(:, k+1) = OF_out(1);
    else
        OF_e(:, k+1) = OF_e(:, k);
    end
    GT_OF_out = OF_output_const_alt_changing_att_moment(x(:, k + 1), 0, 0);
    GT_OF_e(:, k+1) = GT_OF_out(1);
    Cov(:, k+1) = diag(P_k1_k1);
    % Observability calculator
    COi(1, k) = observability_deg_constant_alt_changing_att_moment(x(:, k), moment(k), I, m, kd, g, RATE_MEASUREMENT);
end

%% Processing error metrics and results:

if(controller_type == VELOCITY_REF)
    GT_control_err = x(1,1:end)-control_ref(1,1:end);
    observed_control_err = Xe(1,1:end)-control_ref(1,1:end);
    observed = Xe(1,1:end);
    GT = x(1,1:end);
elseif(controller_type == ATTITUDE_REF)
    GT_control_err = x(2,1:end)-control_ref(1,1:end);
    observed_control_err = Xe(2,1:end)-control_ref(1,1:end);
    observed = Xe(2,1:end);
    GT = x(2,1:end);
else
    GT_control_err = GT_OF_e - control_ref(1,1:end);
    observed_control_err = z(1,:) - control_ref(1,1:end);
    observed = z(1,:);
    GT = GT_OF_e;
end

% Estimation errors:
metrics.MAE_state = mean(abs(Xe - x), 2);
metrics.MAE_OF = mean(abs(GT_OF_e - z(1, :)));

% Control errors:
% MAE during whole run:
metrics.MAE_GT = mean(abs(GT_control_err));
metrics.MAE = mean(abs(observed_control_err));

% During last N seconds
N_sec = 1;
metrics.N_sec = N_sec;
n_steps_sec = round(1/dt);
metrics.MAE_GT_last_N_sec = mean(abs(GT_control_err(end-N_sec*n_steps_sec:end)));
metrics.MAE_last_N_sec = mean(abs(observed_control_err(end-N_sec*n_steps_sec:end)));

% In case of step references:
if(length(params.control_ref) == 2)
    if(params.control_ref(2) > params.control_ref(1))
        POS_PEAKS = true;
    else
        POS_PEAKS = false;
    end
    [rise_time, percentage_overshoot, mean_absolute_error] = get_performance_characteristics( ...
        smooth(GT(round(N/2):end), n_steps_sec), control_ref(end), t(round(N/2):end)-t(round(N/2)), POS_PEAKS); 
    metrics.rise_time = rise_time;
    metrics.percentage_overshoot = percentage_overshoot;
    metrics.mean_absolute_error = mean_absolute_error;
end

% Storing results:
results.state = x;
results.estimated_state = Xe;
results.observability = COi;
results.time = t;
results.rate = x(3,:);

%% Plotting:
if(params.graphics)
    figure('name', 'States and optical flow over time');
    set(gcf, 'Color', [1 1 1]);
    % VELOCITY:
    subplot(2,2,1);
    plot(t(1:end), x(1,1:end), 'LineWidth',1.2);
    hold on;
    plot(t(1:end), Xe(1,1:end), 'LineWidth',1.2);
    if(controller_type == VELOCITY_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth',1.2);
    end
    hold off;
    xlabel('time (s)');
    ylabel('v (m/s)');
    if(controller_type == VELOCITY_REF)
        legend('True', 'Estimated', 'Ref')
    else
        legend('True', 'Estimated')
    end
    set(gca,'FontSize', 16);
    
    % ATTITUDE:
    subplot(2,2,2);
    plot(t(1:end), x(2,1:end)*180/pi, 'LineWidth',1.2)
    hold on;
    plot(t(1:end), Xe(2,1:end)*180/pi, 'LineWidth',1.2)
    if(controller_type == ATTITUDE_REF)
        plot(t(1:end), control_ref(1,1:end)*180/pi, '--k', 'LineWidth',1.2);
    end
    hold off;
    xlabel('time (s)');
    ylabel('\phi (deg)');
    if(controller_type == ATTITUDE_REF)
        legend('True', 'Estimated', 'Ref')
    else
        legend('True', 'Estimated')
    end
    set(gca,'FontSize', 16);
    
    % Height:
    subplot(2,2,3);
    plot(t(1:end), x(4,1:end), 'LineWidth',1.2)
    hold on;
    plot(t(1:end), Xe(4,1:end), 'LineWidth',1.2)
    hold off;
    xlabel('time (s)');
    ylabel('h (m)');
    legend('True', 'Estimated');
    set(gca,'FontSize', 16);
    
    % VENTRAL FLOW:
    subplot(2,2,4);
    plot(t(1:end), z(1, 1:N), 'LineWidth',1.2);
    hold on;
    plot(t(1:end), OF_e(1, 1:end), 'LineWidth',1.2);
    plot(t(1:end), GT_OF_e(1, 1:end), 'LineWidth',1.2);
    if(controller_type == OF_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth',1.2);
    end
    xlim([t(1) t(end)])
    xlabel('time (s)');
    ylabel('Optical flow (rad/s)');
    if(controller_type == OF_REF)
        legend('Measured (delayed)', 'Measured (undelayed)', 'Ground truth (undelayed)', 'Ref');
    else
        legend('Measured (delayed)', 'Measured (undelayed)', 'Ground truth (undelayed)');
    end
    set(gca,'FontSize', 16);
    
    % MOMENT:
    figure('name', 'Moment');
    set(gcf, 'Color', [1 1 1]);
    plot(t(1:end), moment(1, 1:N), 'LineWidth',1.2);
    xlim([t(1) t(end)])
    xlabel('time (s)');
    ylabel('Moment (Nm)');
    legend('Commanded');
    set(gca,'FontSize', 16);
    
    % CONTROLLER_PERFORMANCE:
    figure('name', 'Controller performance');
    set(gcf, 'Color', [1 1 1]);
    if(controller_type == VELOCITY_REF)
        GT_control_err = x(1,1:end)-control_ref(1,1:end);
        observed_control_err = Xe(1,1:end)-control_ref(1,1:end);
    elseif(controller_type == ATTITUDE_REF)
        GT_control_err = x(2,1:end)-control_ref(1,1:end);
        observed_control_err = Xe(2,1:end)-control_ref(1,1:end);
    else
        GT_control_err = GT_OF_e - control_ref(1,1:end);
        observed_control_err = z(1,:) - control_ref(1,1:end);
    end
    plot(t(1:end), GT_control_err, 'LineWidth',1.2);
    hold on;
    plot(t(1:end), observed_control_err, 'LineWidth',1.2);
    xlabel('Time (s)');
    if(controller_type == VELOCITY_REF)
        ylabel('Velocity error (m/s)');
        legend('$v - v_{ref}$', '$\hat{v}-v_{ref}$','Interpreter','latex');
    elseif(controller_type == ATTITUDE_REF)
        ylabel('Attitude error (rad)');
        legend('$\phi - \phi_{ref}$', '$\hat{\phi}-\phi_{ref}$','Interpreter','latex');
    else
        ylabel('Opitcal flow error (1/s)');
        legend('$\omega - \omega_{ref}$', '$\hat{\omega}-\omega_{ref}$','Interpreter','latex');
    end
    set(gca,'FontSize', 16);
    
    % Observability:
    figure('name', 'Observability');
    set(gcf, 'Color', [1 1 1]);
    plot(t(1:end), COi(1, 1:N), 'LineWidth',1.2);
    xlim([t(1) t(end)])
    xlabel('time (s)');
    ylabel('Observability degree (-)');
    legend('Observability degree');
    set(gca,'FontSize', 16);
    
    % Velocity:
    figure('name', 'Velocity over time');
    set(gcf, 'Color', [1 1 1]);
    plot(t(1:end), x(1,1:end), 'LineWidth',1.2);
    hold on;
    plot(t(1:end), Xe(1,1:end), 'LineWidth',1.2);
    if(controller_type == VELOCITY_REF)
        plot(t(1:end), control_ref(1,1:end), '--k', 'LineWidth',1.2);
    end
    hold off;
    xlabel('time (s)');
    ylabel('v (m/s)');
    if(controller_type == VELOCITY_REF)
        legend('True', 'Estimated', 'Ref')
    else
        legend('True', 'Estimated');
    end
    set(gca,'FontSize', 16);
    
    %Kalman gain
    figure('name', 'Kalman gain over time');
    set(gcf, 'Color', [1 1 1]);
    plot(t(1:end), Kalman_gain(1,1:end), 'LineWidth',1.2);
    hold on;
    plot(t(1:end), Kalman_gain(2,1:end), 'LineWidth',1.2);
    plot(t(1:end), Kalman_gain(3,1:end), 'LineWidth',1.2);
    xlabel('time (s)');
    ylabel('v (m/s)');
    legend('K v', 'K \phi', 'K Z')
    set(gca,'FontSize', 16);
end