function sym_observability(CASE)
% function sym_observability(CASE)
%
% Analyze non-linear observability of systems using symbolic variables
%
% Only ventral flow as observation:
% 1: Varying attitude, constant altitude, rate form (Materials and Methods)
% 2: Varying attitude, constant altitude, moment form, measure rate (SM-I-A) 
% 3: Varying attitude, constant altitude, moment form, not measuring rate (SM-I-A)
% 4: Constant altitude, slope, z considered constant
% 5: Full model with wind
%
% Also measuring divergence:
% 6: Rate model for full motion with thrust bias
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Abhishek Chatterjee | abhishek.chaterjee002@gmail.com
% Guido de Croon | g.c.h.e.decroon@tudelft.nl
 

if(~exist('CASE', 'var') || isempty(CASE))
    CASE = 1;
end

% initialize symbolic variables
syms ad a v x xd xdd z w zdd theta thetad M T I m kd g Y y yd ydd yddd thetadd p phi C1 C2 C3 C4 kd A Tb B slope phi_wings p_wings;
assume(ad, 'real');
assume(a, 'real');
assume(v, 'real');
assume(x, 'real');
assume(xd, 'real');
assume(xdd, 'real');
assume(z, 'real');
assume(w, 'real');
assume(zdd, 'real');
assume(theta, 'real');
assume(thetad, 'real');
assume(M, 'real');
assume(T, 'real');
assume(I, 'real');
assume(m, 'real');
assume(kd, 'real');
assume(g, 'real');
assume(g, 'positive');
assume(Y, 'real');
assume(y, 'real');
assume(yd, 'real');
assume(ydd, 'real');
assume(yddd, 'real');
assume(thetadd, 'real');
assume(p, 'real');
assume(phi, 'real');
assume(C1, 'real');
assume(C2, 'real');
assume(C3, 'real');
assume(C4, 'real');
assume(kd, 'real');
assume(kd >= 0);
assume(A, 'real');
assume(B, 'real');
assume(Tb, 'real');
assume(slope, 'real');
assume(phi_wings, 'real');
assume(p_wings, 'real');

if(CASE == 1)
    % ***********************************************
    % Varying attitude, constant altitude, rate form:
    % ***********************************************
    fprintf('Varying attitude, constant altitude, rate form.\n');
    
    
    X = [v phi z]';             % State Vector
    f = [(g*tan(phi)); p; 0];   % State Derivative
    zm =  -v*(cos(phi))^2/z + p;   % State measurement
    
    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1 = simplify(gr'*f);
    gr = gradient(Ld1, X');
    Ld2 = simplify(gr'*f);
    Ld = [zm(1); Ld1; Ld2];
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X'))];
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    % Determine when the observability matrix looses full rank:
    O_phi0 = subs(O, phi, 0);
    fprintf('phi = 0, rank(O) = %f\n', rank(O_phi0));
    O_v0 = subs(O, [v], [0]);
    fprintf('v = 0, rank(O) = %f\n', rank(O_v0)); 
    O_v0_phi0 = subs(O_v0, [phi], [0]);
    fprintf('v = 0, phi = 0, rank(O) = %f\n', rank(O_v0_phi0)); 
    O_p0 = subs(O, [p], [0]);
    fprintf('p = 0, rank(O) = %f\n', rank(O_p0)); 
    
elseif(CASE == 2)
    % *****************************************************************
    % Varying attitude, constant altitude, moment form, measuring rate:
    % *****************************************************************
    fprintf('Varying attitude, constant altitude, moment form, measuring rate.\n');
    
    X = [v phi p z]';             % State Vector
    f = [(g*tan(phi)); p; M/I; 0];   % State Derivative
    zm = [-v*(cos(phi))^2/z + p, p];   % State measurement
    
    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1 = simplify(gr'*f);
    gr = gradient(Ld1, X');
    Ld2 = simplify(gr'*f);
    % Watch out! With rate measurements, this becomes a choice!
    Ld = [zm(1); Ld1; Ld2; zm(2)];
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X')), ...
        simplify(gradient(Ld(4), X'))]; %'
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    % What if p, M are 0?
    O_M0_p0 = subs(O, [p, M], [0,0]);
    fprintf('Rank O_M0_p0 = %f\n', rank(O_M0_p0)); 
    O_p0 = subs(O, [p], [0]);
    fprintf('Rank O_p0 = %f\n', rank(O_p0)); 
    O_M0 = subs(O, [M], [0]);
    fprintf('Rank O_M0 = %f\n', rank(O_M0)); 
    O_phi0 = subs(O, phi, 0);
    fprintf('Rank O_phi0 = %f\n', rank(O_phi0)); 
    O_M0_phi0 = subs(O, [M, phi], [0, 0]);
    fprintf('Rank O_M0_phi0 = %f\n', rank(O_M0_phi0)); % In the slice of Fig. 1, M = 0.
    O_v0 = subs(O, [v], [0]);
    fprintf('Rank O_v0 = %f\n', rank(O_v0)); 
    O_v0_phi0 = subs(O_v0, [phi], [0]);
    fprintf('Rank O_v0_phi0 = %f\n', rank(O_v0_phi0)); 
    
elseif(CASE == 3)
    % *********************************************************************
    % Varying attitude, constant altitude, moment form, not measuring rate:
    % *********************************************************************
    fprintf('Varying attitude, constant altitude, moment form, not measuring rate');
    
    X = [v phi p z]';             % State Vector
    f = [(g*tan(phi)); p; M/I; 0];   % State Derivative
    zm = -v*(cos(phi))^2/z + p;   % State measurement
    
    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1 = simplify(gr'*f);
    gr = gradient(Ld1, X');
    Ld2 = simplify(gr'*f);
    gr = gradient(Ld2, X');
    Ld3 = simplify(gr'*f);
    Ld = [zm(1); Ld1; Ld2; Ld3];
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X')), ...
        simplify(gradient(Ld(4), X'))]; %'
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    % What if p, M are 0?
    O_M0_p0 = subs(O, [p, M], [0,0]);
    fprintf('Rank O_M0_p0 = %f\n', rank(O_M0_p0)); 
    O_p0 = subs(O, [p], [0]);
    fprintf('Rank O_p0 = %f\n', rank(O_p0)); 
    O_M0 = subs(O, [M], [0]);
    fprintf('Rank O_M0 = %f\n', rank(O_M0)); 
    O_phi0 = subs(O, phi, 0);
    fprintf('Rank O_phi0 = %f\n', rank(O_phi0)); 
    O_M0_phi0 = subs(O, [M, phi], [0, 0]);
    fprintf('Rank O_M0_phi0 = %f\n', rank(O_M0_phi0)); 
    O_v0 = subs(O, [v], [0]);
    fprintf('Rank O_v0 = %f\n', rank(O_v0)); 
    O_v0_phi0 = subs(O_v0, [phi], [0]);
    fprintf('Rank O_v0_phi0 = %f\n', rank(O_v0_phi0)); 

elseif(CASE == 4)
    
    % ********************************************
    % Varying attitude, constant altitude + slope:
    % ********************************************
    fprintf('Varying attitude, constant altitude, rate form + slope.\n');
    
    % slope = slope *angle*
    X = [v phi z slope]';             % State Vector
    f = [(g*tan(phi)); p; v*tan(slope); 0];   % State Derivative
    zm = (-v*cos(phi)*cos(phi + slope)) / (z * cos(slope)) + p;   % State measurement
    
    % Determine Lie derivatives:
    % ventral flow:
    gr = gradient(zm(1), X');
    Ld1_1 = simplify(gr'*f);    
    gr = gradient(Ld1_1, X');
    Ld1_2 = simplify(gr'*f);
    gr = gradient(Ld1_2, X');
    Ld1_3 = simplify(gr'*f);
    
    Ld = [zm(1); Ld1_1; Ld1_2; Ld1_3]; % only ventral flow
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X')), ...
        simplify(gradient(Ld(4), X'))]';
       
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
elseif(CASE == 5)
    % ************************
    % 5: Full model with wind:
    % ************************
    
    X = [v phi p z w A B]';                 % State Vector
    % Linear drag model:
    % f = [T*sin(phi)/m - kd*(v-A)/m; p; M/I; w; T*cos(phi)/m-g-kd*(w-B)/m; 0; 0];   % State Derivative with drag and wind
    % Normal drag model:
    f = [T*sin(phi)/m - kd*(v-A)^2/m; p; M/I; w; T*cos(phi)/m-g-kd*(w-B)^2/m; 0; 0];   % State Derivative with drag and wind
    % f = [T*sin(phi)/m - sign(v-A) * kd*(v-A)^2/m; p; M/I; w; T*cos(phi)/m-g-sign(w-B)*kd*(w-B)^2/m; 0; 0];   % State Derivative with drag and wind
    assume(v-A >= 0); % we can assume this without loss of generality, since
                          % the model is symmetric, and we study local
                          % observability here.
    assume(w-B >= 0); % This restricts the wind to a quadrant, so also the opposite condition has to be checked (below)
    
    zm = -v*(cos(phi))^2/z + w*(sin(2*phi))/(2*z)+p;   % State measurements

    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1_1 = simplify(gr'*f);
    gr = gradient(Ld1_1, X');
    Ld2_1 = simplify(gr'*f);
    gr = gradient(Ld2_1, X');
    Ld3_1 = simplify(gr'*f);
    gr = gradient(Ld3_1, X');
    Ld4_1 = simplify(gr'*f);
    gr = gradient(Ld4_1, X');
    Ld5_1 = simplify(gr'*f);
    gr = gradient(Ld5_1, X');
    Ld6_1 = simplify(gr'*f);    
    Ld = [zm(1); Ld1_1; Ld2_1; Ld3_1; Ld4_1; Ld5_1; Ld6_1];

    % Simplification takes a long time, so we leave it out for fast
    % execution. A simplified version can be obtained like this:
    % Observability matrix:
    %     O = [simplify(gradient(Ld(1), X')), ...
    %         simplify(gradient(Ld(2), X')), ...
    %         simplify(gradient(Ld(3), X')), ...
    %         simplify(gradient(Ld(4), X')), ...
    %         simplify(gradient(Ld(5), X')), ...
    %         simplify(gradient(Ld(6), X')), ...
    %         simplify(gradient(Ld(7), X'))]';
    
    O = [gradient(Ld(1), X'), ...
        gradient(Ld(2), X'), ...
        gradient(Ld(3), X'), ...
        gradient(Ld(4), X'), ...
        gradient(Ld(5), X'), ...
        gradient(Ld(6), X'), ...
        gradient(Ld(7), X')]';
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    % What conditions lead to unobservability?
    O_M0_p0 = subs(O, [p, M], [0,0]);
    fprintf('Rank O_M0_p0 = %f\n', rank(O_M0_p0));
    O_p0 = subs(O, [p], [0]);
    fprintf('Rank O_p0 = %f\n', rank(O_p0)); 
    O_M0 = subs(O, [M], [0]);
    fprintf('Rank O_M0 = %f\n', rank(O_M0)); 
    O_const_alt = subs(O, [w, T], [0, m*g/cos(phi)]);
    fprintf('rank(O_const_alt) = %f\n', rank(O_const_alt));
    O_const_alt_p0 = subs(O_const_alt, p, 0);
    fprintf('rank(O_const_alt_p0) = %f\n', rank(O_const_alt_p0));
    O_const_alt_p0_phi0 = subs(O_const_alt, [p, phi], [0, 0]);
    fprintf('rank(O_const_alt_p0_phi0) = %f\n', rank(O_const_alt_p0_phi0));
    O_p0_phi0 = subs(O, [p, phi], [0, 0]);
    fprintf('rank(O_p0_phi0) = %f\n', rank(O_p0_phi0));
    O_Tgrav = subs(O, [T], [m*g/cos(phi)]);
    fprintf('rank(O_Tgrav) = %f\n', rank(O_Tgrav));
    O_Tgrav_p0 = subs(O_Tgrav, [p], [0]);
    fprintf('rank(O_Tgrav_p0) = %f\n', rank(O_Tgrav_p0));
    O_Tgrav_phi0 = subs(O_Tgrav, [phi], [0]);
    fprintf('rank(O_Tgrav_phi0) = %f\n', rank(O_Tgrav_phi0));
    O_Tgrav_p0_phi0 = subs(O_Tgrav, [p, phi], [0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0) = %f\n', rank(O_Tgrav_p0_phi0));
    O_Tgrav_p0_phi0_v0 = subs(O_Tgrav, [p, phi, v], [0, 0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0_v0) = %f\n', rank(O_Tgrav_p0_phi0_v0));
    O_Tgrav_p0_M0 = subs(O_Tgrav, [p, M], [0, 0]);
    fprintf('rank(O_Tgrav_p0_M0) = %f\n', rank(O_Tgrav_p0_M0));
    O_Tgrav_p0_M0_phi0 = subs(O_Tgrav, [p, M, phi], [0, 0, 0]);
    fprintf('rank(O_Tgrav_p0_M0_phi0) = %f\n', rank(O_Tgrav_p0_M0_phi0));
    O_Tgrav_p0_phi0_v0_M0 = subs(O_Tgrav, [p, phi, v, M], [0, 0, 0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0_v0_M0) = %f\n', rank(O_Tgrav_p0_phi0_v0_M0));
    O_Tgrav_p0_phi0_v0_M0_w0 = subs(O_Tgrav, [p, phi, v, M, w], [0, 0, 0, 0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0_v0_M0_w0) = %f\n', rank(O_Tgrav_p0_phi0_v0_M0_w0));   
    O_Tgrav_p0_phi0_v0_M0_wB = subs(O_Tgrav, [p, phi, v, M, w], [0, 0, 0, 0, B]);
    fprintf('rank(O_Tgrav_p0_phi0_v0_M0_wB) = %f\n', rank(O_Tgrav_p0_phi0_v0_M0_wB));   
    O_Tgrav_p0_phi0_vA_M0_w0 = subs(O_Tgrav, [p, phi, v, M, w], [0, 0, A, 0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0_vA_M0_w0) = %f\n', rank(O_Tgrav_p0_phi0_vA_M0_w0));
    O_Tgrav_p0_phi0_vA_M0_wB = subs(O_Tgrav, [p, phi, v, M, w], [0, 0, A, 0, B]);
    fprintf('rank(O_Tgrav_p0_phi0_vA_M0_wB) = %f\n', rank(O_Tgrav_p0_phi0_vA_M0_wB));
    
elseif(CASE == 6)
    % ****************************************************************
    % Varying attitude, varying altitude, rate form, with thrust bias:
    % ****************************************************************
    
    X = [v phi z w Tb]'; % State Vector
    f = [(T-Tb)*sin(phi)/m-kd*v/m; p; w; (T-Tb)*cos(phi)/m-g; 0]; % State Derivative (with linear drag model)
    
    % Measure both ventral flow and divergence:
    zm = [-v*(cos(phi))^2/z + w*(sin(2*phi))/(2*z)+p, -(v*sin(2*phi))/(2*z)-(w*cos(phi)*cos(phi))/z];   % State measurements
    
    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1_1 = gr'*f;
    gr = gradient(zm(2), X');
    Ld1_2 = gr'*f;
    gr = gradient(Ld1_1, X');
    Ld2_1 = gr'*f;
    gr = gradient(Ld1_2, X');
    Ld2_2 = gr'*f;
    
    % Watch out! With divergence measurements, this becomes a choice!
    Ld = [zm(1); zm(2); Ld1_1; Ld1_2; Ld2_1];
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X')), ...
        simplify(gradient(Ld(4), X')), ...
        simplify(gradient(Ld(5), X'))]';
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    % Study conditions that make it unobservable:
    O_const_alt = subs(O, [w, T], [0, m*g/cos(phi)]);
    fprintf('rank(O_const_alt) = %f\n', rank(O_const_alt));
    O_const_alt_p0 = subs(O_const_alt, p, 0);
    fprintf('rank(O_const_alt_p0) = %f\n', rank(O_const_alt_p0));
    O_const_alt_p0_phi0 = subs(O_const_alt, [p, phi], [0, 0]);
    fprintf('rank(O_const_alt_p0_phi0) = %f\n', rank(O_const_alt_p0_phi0));
    O_p0_phi0 = subs(O, [p, phi], [0, 0]);
    fprintf('rank(O_p0_phi0) = %f\n', rank(O_p0_phi0));
    O_Tgrav = subs(O, [T], [m*g/cos(phi)]);
    fprintf('rank(O_Tgrav) = %f\n', rank(O_Tgrav));
    O_Tgrav_p0 = subs(O_Tgrav, [p], [0]);
    fprintf('rank(O_Tgrav_p0) = %f\n', rank(O_Tgrav_p0));
    O_Tgrav_phi0 = subs(O_Tgrav, [phi], [0]);
    fprintf('rank(O_Tgrav_phi0) = %f\n', rank(O_Tgrav_phi0));
    O_Tgrav_p0_phi0 = subs(O_Tgrav, [p, phi], [0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0) = %f\n', rank(O_Tgrav_p0_phi0));
    O_Tgrav_p0_phi0_v0 = subs(O_Tgrav, [p, phi, v], [0, 0, 0]);
    fprintf('rank(O_Tgrav_p0_phi0_v0) = %f\n', rank(O_Tgrav_p0_phi0_v0));
    O_Tgrav_p0_v0 = subs(O_Tgrav, [p, v], [0, 0]);
    fprintf('rank(O_Tgrav_p0_v0) = %f\n', rank(O_Tgrav_p0_v0));
elseif(CASE == 7)
    
    % ***************************************************************************
    % Varying attitude, constant altitude, rate form, insect head and body model:
    % ***************************************************************************
    fprintf('Varying attitude, constant altitude, rate form, insect head and body model.\n');
    
    X = [v phi phi_wings z]';             % State Vector
    f = [(g*tan(phi_wings)); p; p_wings; 0];   % State Derivative
    zm = [-v*(cos(phi))^2/z + p; phi - phi_wings];   % State measurements
    
    % Determine Lie derivatives:
    gr = gradient(zm(1), X');
    Ld1 = simplify(gr'*f);
    gr = gradient(Ld1, X');
    Ld2 = simplify(gr'*f);
    gr = gradient(Ld2, X');
    Ld3 = simplify(gr'*f);
    
    Ld = [zm(1); Ld1; Ld2; zm(2)];
    % Ld = [zm(1); Ld1; Ld2; Ld3]; 
    
    % Observability matrix:
    O = [simplify(gradient(Ld(1), X')), ...
        simplify(gradient(Ld(2), X')), ...
        simplify(gradient(Ld(3), X')), ...
        simplify(gradient(Ld(4), X'))]; %'
    
    % Full rank means observable:
    fprintf('Rank of O = %d.\n', rank(O));
    if(length(X) == rank(O))
        fprintf('Observable system.\n');
    else
        fprintf('Unobservable system.\n');
    end
    
    O_p0 = subs(O, [p], [0]);
    fprintf('Rank O_p0 = %f\n', rank(O_p0)); 
    O_pwings0 = subs(O, [p_wings], [0]);
    fprintf('Rank O_pwings0 = %f\n', rank(O_pwings0));
    O_p0_pwings0 = subs(O, [p, p_wings], [0, 0]);
    fprintf('Rank O_p0_pwings0 = %f\n', rank(O_p0_pwings0));
end
