function verify_hypothesis_faster_flight_insect(N, GT_MODEL)
% function verify_hypothesis_faster_flight_insect(N, GT_MODEL)
%
% Article: 
% Accommodating unobservability to control flight attitude with optic flow
% G.C.H.E. de Croon, J.J.G. Dupeyroux, C. De Wagter, A. Chatterjee, D.A. Olejnik, F. Ruffier
%
% Code by:
% Guido de Croon | g.c.h.e.decroon@tudelft.nl

if(~exist('N', 'var') || isempty(N))
    N = 5;
end
if(~exist('GT_MODEL', 'var') || isempty(GT_MODEL))
    GT_MODEL = false;
end

% set the random seed generator:
rng(2021)

% set parameters for the simulations:
params = set_standard_params_simulate_controller_constant_altitude(false, false, true);
params.OFdt = 1/30;
params.sigma_OF = 0.02;
params.delay_OF = 0.033;
params.sigma_rate = 0.005;
params.sigma_T = 0.05; % disturbances
params.sigma_head_thorax = deg2rad(0.5); % measurement noise of angle between head and body

% delays, all in seconds:
params.delay_rate = 0.01;      % delay in rate input to system
params.delay_rate_k = 0.01;   % delay in rate input to kalman filter
params.delay_T = 0.01;      % delay in thrust input to system
params.delay_T_k = 0.01;   % delay in thrust input to kalman filter
params.delay_ht = 0.01;    % delay in head-thorax measurement

% External disturbances:
NO_DISTURBANCE = 0;
THRUST_DISTURBANCE = 1;
HORIZONTAL_DISTURBANCE = 2;
MALICIOUS_V0 = 3;
params.external_disturbance = HORIZONTAL_DISTURBANCE; 

% whether to include actuator dynamics in case of rate commands:
params.ACTUATOR_DYNAMICS = true;

% Include divergence as measurement:
params.DIVERGENCE = true;
params.R = diag([0.1, 0.1, 0.001].^2);

% drag parameters:
params.kd = 2.0; 
params.WING_DRAG = true; 

% Control gains:
params.K_v = 50*pi/180;
params.K_i = 0.0*params.K_v; 
params.K_theta = 2.0; 

% Actuation properties:
params.alpha = 0.2; 
params.delay_rate = 0.09; 
params.sigma_OF = 0.02; 

if(GT_MODEL)
    % Trend with ground truth (and additive noise) instead of estimation:
    params.USE_NOISY_GROUND_TRUTH = true;
    params.std_GT_attitude_noise = deg2rad(0.25);
else
    % Trend when estimating attitude
    params.USE_NOISY_GROUND_TRUTH = false;
    params.std_GT_attitude_noise = deg2rad(0.25);
end

% Don't plot intermediate graphics:
params.graphics = false;

% Reference optic flow values at 1 m:
references = 0.1:0.1:0.8;
n_refs = size(references, 2);

% Initializing all variables to be tracked over the runs:
STDs = zeros(n_refs, 1);
OBSs = zeros(n_refs, 1);
PHIs  = cell(n_refs, 1);
STD_PHIs = zeros(n_refs, 1);
MEAN_PHIs = zeros(n_refs, 1);
STD_PHIs_HEAD = zeros(n_refs, 1);
MEAN_PHIs_HEAD = zeros(n_refs, 1);
MAE_ESTIMATION = cell(n_refs, 1);
for r = 1:n_refs
    params.control_ref = [references(r)];
    [mean_MAE, mean_STD, mean_OBS, STD_PHI, OBS, PHI, STD_PHI_HEAD, PHI_HEAD, ESTIMATION_ERRORS] = ...
        get_results_of_setting(N, params);
    STDs(r) = mean_STD;
    OBSs(r) = mean_OBS;
    PHIs{r} = PHI;
    fPHI = flatten_cell(PHI);
    STD_PHIs(r) = std(fPHI);
    MEAN_PHIs(r) = mean(fPHI);
    fPHI_HEAD = flatten_cell(PHI_HEAD);
    STD_PHIs_HEAD(r) = std(fPHI_HEAD);
    MEAN_PHIs_HEAD(r) = mean(fPHI_HEAD);
    MAE_ESTIMATION{r} = get_average_cell(ESTIMATION_ERRORS);
    fprintf('STD at %f = %f\n', references(r), STDs(r));
end

results.STDs = STDs;
results.OBSs = OBSs;
results.PHIs = PHIs;
results.STD_PHIs = STD_PHIs;
results.MEAN_PHIs = MEAN_PHIs;
results.references = references;
results.MEAN_PHIs_HEAD = MEAN_PHIs_HEAD;
results.STD_PHIs_HEAD = STD_PHIs_HEAD;
results.MAE_ESTIMATION = MAE_ESTIMATION;
if(GT_MODEL)
    save('results_speed_hypothesis_GT', 'results');
else
    save('results_speed_hypothesis', 'results');
end



function [mean_MAE, mean_STD, mean_OBS, STD_PHI, OBS, PHI, STD_PHI_HEAD, PHI_HEAD, ESTIMATION_ERRORS] = ...
            get_results_of_setting(N, params)

    verbose = true;
    if(N == 1)
        params.graphics = true;
    end
    
    % initialize variables for storing results over multiple runs
    MAE_GT = zeros(N, 1);
    STD_PHI = zeros(N, 1);
    OBS = zeros(N, 1);
    PHI = cell(N,1);
    STD_PHI_HEAD = zeros(N, 1);
    PHI_HEAD = cell(N, 1);
    ESTIMATION_ERRORS = cell(N,1);
    for exp = 1:N
        fprintf('Experiment %d\n', exp);
        [metrics, results] = simulate_controller_constant_altitude_changing_attitude_insect(params);
        
        phi = results.state(3, :);
        phi = rad2deg(phi(round(end/2):end)); 
        PHI{exp} =  phi;
        STD_PHI(exp) = std(phi);
        
        phi_head = results.state(2, :);
        phi_head = rad2deg(phi_head(round(end/2):end)); 
        PHI_HEAD{exp} = phi_head;
        STD_PHI_HEAD(exp) = std(phi_head);
        
        observ = results.observability;
        observ = observ(round(end/2):end);
        OBS(exp) = mean(observ);
        if(verbose)
            print_metrics(metrics);
        end
        
        MAE_GT(exp) = metrics.MAE_GT;
        
        
        ESTIMATION_ERRORS{exp} = mean(abs(results.estimated_state(:,round(end/2):end) - results.state(:,round(end/2):end)), 2);
    end
    
    
    mean_MAE = mean(MAE_GT);
    mean_STD = mean(STD_PHI);
    mean_OBS = mean(OBS);